# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom,
#                          and Karl MacMillan
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import traceback
from sys import exc_info
from inspect import isfunction
from gamera import group, util
from fudge import Fudge

"""This module provides tools for applying graph-rewriting rules to a
set of glyphs.

It would seem nice to invent a completely new language for this task
(for instance, a logic-programming language like Prolog might have
been nice).

However, in this design, each rule is defined by a basic Python
function. This integrates much easier into the rest of the
Python-based Gamera world, since any Python function can be called at
any time, and doesn't require the user to learn a new language.  The
disadvantage is that the pattern matching is single-tiered (no
recursion) and the ordering and application of the rules is left to
the end programmer, since this approach makes it difficult to do any
global optimisation.  There may be ways around these limitations, but
that will require some thinking.

Rule functions take the basic form:

   def rule_function0(a="dot", b="letter*"):      # 1: function signature
      if (Fudge(a.lr_y) == b.lr_y and
          a.ul_x > b.lr_x and
          a.ul_x - b.lr_x < 20):                   # 2: expression
         a.classify_heuristic('punctuation.period') # 3: operation
         return [], []                              # 4: added, removed

1. The function signature itself is used to define which classes of
   glyphs should be applied to the function.  Each argument has a
   Python default value which is a string containing the symbol name
   or an "symbol name regular expression" that will match the glyph to
   be passed in.  Yes, this is a complete bastardisation of what
   default arguments are meant for in Python, since these string
   values will never actually be passed in to the function.  However,
   I think this syntax is convenient and kind of cute.  (Another
   option might be to use some sort of special doc-string syntax, I
   suppose.)

   Rules are indexed by the first argument, so it is a good idea to
   use the least-frequently occuring symbol name (or the class the
   most clearly defines the structure) as the first argument.

   A "symbol name regular expression" is an instance of the
   special-purpose regular expression language for symbol names
   defined in id_name_matching.py.  This syntax is pretty limited
   compared to standard Python regular expression syntax, but is much
   more convenient for the task. (For example, 'upper.*' would be
   'upper\.[^.]*' as a standard Python regular expression, and nobody
   wants to write code like that ;)

      Informal syntax definition:
         A|B                          # matches A or B
         A.B|C                        # matches A.B or A.C 
         (A.B)|C                      # matches A.B or C
         *                            # multiple-character wildcard
         ?                            # single character wildcard
         ()                           # grouping can be performed with
            # parentheses
         [a-z]                        # matches any character a-z

      Example expressions:
         (upper.x)|(lower.y)          # match either upper.x or lower.y
         upper.*                      # match the 'upper' category
         upper.a|b|c                  # matches upper.a, upper.b or upper.c
         upper.capital_?              # ? is a single character wildcard

2. The body of the function, in a literal sense, is left entirely up
   to the end-user programmer (it's just Python afterall.)  However,
   most rule functions will consist of some sort of test followed by
   some sort of change (side-effect!!!) of the data.

   For performing tests that need to be somewhat "fuzzy", you may want
   to use Fudge objects.  Comparisons on Fudge objects have a built-in
   fuzzy zone around the core int, Point or Rect that make comparing
   things to be "close enough" easier.  (See fudge.py)

3. In the body, the rule function can modify the glyph attributes in
   any way it sees fit.

4. Optionally, the rule function can return a pair of lists.  The
   first list is of glyphs that have been created by the operation
   (for instance, if combining two glyphs into one union glyph.)  The
   second list is a list of glyphs that have been removed by the
   operation (for instance removing noise glyphs.)  (If your function
   returns None, the RuleEngine assumes you meant [], []). These
   results are appended together each time a set of rules is applied
   and returned to the caller. Optionally, the rule-performing
   mechanism can reapply rules to all the newly-created glyphs.

PERFORMING RULES:

Rules are grouped into RuleEngine classes, which then perform the
actually searching and call the rule functions.

   # Initialise a new rule engine with a couple of rules
   rule_engine = RuleEngine([rule_function0, rule_function1])
   # Run the rules on the given list of glyphs
   added, removed = rule_engine.perform_rules(glyphs)

An optional argument, grid_size, specifies the size of the indexing
grid cells.  This will affect how far apart symbols can be matched.

TODO: There may be at some point some kind of Meta-RuleEngine for
performing sets of rules in sequence or parallel or whatever...  """

class RuleEngineError(Exception):
   pass

class RuleEngine:
   _class_rules = []

   def __init__(self, rules=[], reapply=0):
      self.rules = {}
      self._regexs = {}
      self._rules_by_regex = {}
      for rule in rules + self._class_rules:
         if isfunction(rule):
            self.add_rule(rule)
         elif isinstance(rule, RuleEngine):
            for r in rule.rules:
               self.add_rule(r)
      self._reapply = reapply

   def add_rule(self, rule):
      import id_name_matching
      assert len(rule.func_defaults)
      self.rules[rule] = None
      regex = rule.func_defaults[0]
      if not self._rules_by_regex.has_key(regex):
         self._rules_by_regex[regex] = []
      self._rules_by_regex[regex].append(rule)
      for regex in rule.func_defaults:
         if not self._regexs.has_key(regex):
            self._regexs[regex] = id_name_matching.build_id_regex(regex)
         if not self._rules_by_regex.has_key(regex):
            self._rules_by_regex[regex] = []

   def get_rules(self):
      return self.rules.keys()

   def _deal_with_result(self, rule, glyphs, added, removed):
      try:
         current_stack = traceback.extract_stack()
         result = rule(*glyphs)
      except Exception, e:
         lines = traceback.format_exception(*exc_info())
         del lines[1]
         exception = ''.join(lines)
         # We build a dictionary of exceptions in order to remove duplicates
         self._exceptions.append(exception)
         return 0
      if result is None or result == ([], []):
         return 0
      for a in result[0]:
         added[a] = None
      for a in result[1]:
         removed[a] = None
      return 1

   def perform_rules(self, glyphs, grid_size=100, recurse=0,
                     progress=None, _recursion_level=0):
      self._exceptions = util.Set()
      if _recursion_level > 10:
         return [], []
      elif _recursion_level == 0:
         progress = util.ProgressFactory("Performing rules...")

      try:
         grid_index = group.GridIndexWithKeys(glyphs, grid_size, grid_size)
         found_regexs = {}
         for regex_string, compiled in self._regexs.items():
            for glyph in glyphs:
               if glyph.match_id_name(compiled):
                  grid_index.add_glyph_by_key(glyph, regex_string)
                  found_regexs[regex_string] = None

         # This loop is only so the progress bar can do something useful.
         for regex in found_regexs.iterkeys():
            progress.add_length(
              len(self._rules_by_regex[regex]) *
              len(grid_index.get_glyphs_by_key(regex)))

         added = {}
         removed = {}
         for regex in found_regexs.iterkeys():
            for rule in self._rules_by_regex[regex]:
               glyph_specs = rule.func_defaults
               for glyph in grid_index.get_glyphs_by_key(regex):
                  if len(glyph_specs) == 1:
                     self._deal_with_result(rule, (glyph,), added, removed)
                  elif len(glyph_specs) == 2:
                     for glyph2 in grid_index.get_glyphs_around_glyph_by_key(
                       glyph, glyph_specs[1]):
                        stop = self._deal_with_result(rule, (glyph, glyph2), added, removed)
                        if not self._reapply and stop:
                           break
                  else:
                     seed = [list(grid_index.get_glyphs_around_glyph_by_key(glyph, x))
                             for x in glyph_specs[1:]]
                     for combination in util.combinations(seed):
                        stop = self._deal_with_result(rule, [glyph] + combination,
                                                      added, removed)
                        if not self._reapply and stop:
                           break
                  progress.step()
      finally:
         if _recursion_level == 0:
            progress.kill()
      if recurse and len(added):
         self._deal_with_result(
           self.perform_rules(added.keys(), 1, progress, _recursion_level + 1))

      if len(self._exceptions):
         s = ("One or more of the rule functions caused an exception.\n" +
              "(Each exception listed only once):\n\n" +
              "\n".join(self._exceptions))
         raise RuleEngineError(s)

      return added.keys(), removed.keys()



#
#
# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
#               2010      Christoph Dalitz
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""The deformations module contains plugins for applying
deformations to images."""

from gamera.plugin import *
try:
  from gamera.core import RGBPixel
except Exception:
  def RGBPixel(*args):
    pass

try:
    from gamera.core import *
except Exception:
    pass

import _deformation

class wave(PluginFunction):
    """
    Causes periodic disturbance of user-defined frequency, amplitude,
    and direction.  Turbulence specifies the amount of random
    variation from that line.
    """
    self_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    return_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    args = Args([Int("amplitude"),
                 Int("period"),
                 Choice('direction', ['Horizontal','Vertical']),
                 Choice('waveform', ['Sinusoid','Square','Sawtooth','Triangle','Sinc']),
                 Int('offset'),
                 Float('turbulence', default=0.0),
                 Int('random_seed', default=-1)
                ])
    args.list[0].rng = (0,sys.maxint)
    args.list[1].rng = (0,sys.maxint)
    def __call__(self, amplitude, period, direction, waveform_type=0, offset=0, turbulence=0.0, random_seed=0):
        return _deformation.wave(self, amplitude, period, direction, waveform_type,
                                 offset, turbulence, random_seed)
    __call__ = staticmethod(__call__)
    doc_examples = [(RGB, 5, 10, 0, 0, 0), (RGB, 10, 5, 1, 2, 0)]

class noise(PluginFunction):
    """
    Causes random shifting of pixels within a user-specified range, in
    a user-specified direction.
    """
    self_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    return_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    args = Args([Int("amplitude", range=(0, 500)),
                 Choice('direction',['Horizontal','Vertical']),
                 Int("random_seed", default=-1)
                ])
    doc_examples = [(RGB, 10, 0)]
    def __call__(self, amplitude, direction, random_seed=0):
      return _deformation.noise(self, amplitude, direction, random_seed)
    __call__ = staticmethod(__call__)

class inkrub(PluginFunction):
    """
    Simulates rubbing off of ink from another page.
    """
    self_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    return_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    args = Args([Int("transcription_prob", range=(0, 500)),
                 Int("random_seed", default=-1)])
    doc_examples = [(GREYSCALE, 50)]
    def __call__(self, transcription_prob, random_seed=0):
      return _deformation.inkrub(self, transcription_prob, random_seed)
    __call__ = staticmethod(__call__)

class ink_diffuse(PluginFunction):
    """
    Simulates water-driven diffusion of ink in paper.
    """
    self_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    return_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
    args = Args([Choice('diffusion_type',
                        ["Linear Horizontal","Linear Vertical","Brownian"]),
                 Float("exponential_decay_constant"),
                 Int("random_seed", default=-1)])
    doc_examples = [(GREYSCALE, 0, 20)]
    def __call__(self, diffusion_type, exponential_decay_constant, random_seed=0):
      return _deformation.ink_diffuse(self, diffusion_type, exponential_decay_constant, random_seed)
    __call__ = staticmethod(__call__)

class degrade_kanungo(PluginFunction):
    """Degrades an image due to a scheme proposed by Kanungo et al.
(see the reference below). This is supposed to emulate image defects
introduced through printing and scanning.

The degradation scheme depends on six parameters *(eta,a0,a,b0,b,k)* with
the following meaning:

  - each foreground pixel (black) is flipped with probability
    `a0*exp(-a*d^2) + eta`, where d is the distance to the closest
    background pixel

  - each background pixel (white) is flipped with probability
    `b0*exp(-b*d^2) + eta`, where d is the distance to the closest
    foreground pixel

  - eventually a morphological closing operation is performed with a disk
    of diameter *k*. If you want to skip this step set *k=0*; in that
    case you should do your own smoothing afterwards.

The random generator is initialized with *random_seed* for allowing
reproducible results.

References:

  T. Kanungo, R.M. Haralick, H.S. Baird, et al.:
  *A statistical, nonparametric methodology for document
  degradation model validation.*
  IEEE Transactions on Pattern Analysis and Machine Intelligence
  22, pp. 1209-1223 (2000)
"""
    self_type = ImageType([ONEBIT])
    args = Args([Float('eta', range=(0.0,1.0)),
                 Float('a0', range=(0.0,1.0)),
                 Float('a'),
                 Float('b0', range=(0.0,1.0)),
                 Float('b'),
                 Int('k', default=2),
                 Int('random_seed', default=0)])
    return_type = ImageType([ONEBIT])
    author = "Christoph Dalitz"
    doc_examples = [(ONEBIT, 0.0, 0.5, 0.5, 0.5, 0.5, 2, 0)]

class white_speckles(PluginFunction):
    """Adds white speckles to an image. This is supposed to emulate
image defects introduced through printing, scanning and thresholding.

The degradation scheme depends on three parameters *(p,n,k)* with
the following meaning:

 - Each black pixel in the input image is taken with probability *p*
   as a starting point for a random walk of length *n*.
   Consequently *p* can be interpreted as the speckle frequency and *n*
   as a measure for the speckle size.

 - An image containing the random walk is smoothed by a closing operation
   with a rectangle of size *k*.

 - Eventually the image with the random walks is subtracted from the input
   image, which results in white speckles at the random walk positions

Input arguments:

 *p, n, k*:
   speckle frequency, random walk length and closing disc size

 *connectivity*:
   effects the connectivity of adjacent random walk points as shown
   in the following figure (in case of bishop moves the final closing
   operation whitens the neighbouring 4-connected pixels too):

   .. image:: images/randomwalk_connectivity.png

References:

  C. Dalitz, M. Droettboom, B. Pranzas, I. Fujinaga:
  *A Comparative Study of Staff Removal Algorithms.*
  IEEE Transactions on Pattern Analysis and Machine Intelligence 30,
  pp. 753-766 (2008)
"""
    self_type = ImageType([ONEBIT])
    args = Args([Float('p', range=(0.0,1.0)),
                 Int('n'),
                 Int('k', default=2),
                 Choice('connectivity', ['rook','bishop','king'], default=2),
                 Int('random_seed', default=0)])
    return_type = ImageType([ONEBIT], 'white_speckles')
    author = "Christoph Dalitz"
    doc_examples = [(ONEBIT, 0.05, 5, 2, 2, 0)]

# class batch_deform(PluginFunction):
#     """
#     Performs all possible deformations over given range of arguments.
#     """
#     pure_python = 1
#     self_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])
#     return_type = ImageType([ONEBIT, GREYSCALE, GREY16, FLOAT, RGB])


class DefModule(PluginModule):
    cpp_headers=["deformations.hpp"]
    category = "Deformations"
    functions = [noise, inkrub, wave, ink_diffuse,
                 degrade_kanungo, white_speckles]
    author = "Albert Brzeczko"
    url = "http://gamera.sourceforge.net/"
module = DefModule()




# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom,
#                         and Karl MacMillan
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import wx
import keyword, re

from gamera.gui import gui_util

variable_name = re.compile("^[A-Za-z_][A-Za-z0-9_]*$")
def verify_variable_name(name):
   if keyword.iskeyword(name):
      return 0
   if variable_name.match(name):
      return 1
   return 0

def get(default='untitled', dict={}):
   number = 0
   while 1:
      name = '%s%d' % (default, number)
      while dict.has_key(name):
         number = number + 1
         name = '%s%d' % (default, number)
         if number > 1000:
            number = 0
            break
      dlg = wx.TextEntryDialog(
          None,
          'Please enter a variable name for the result',
          'Select a variable name', name,
          style=wx.OK|wx.CANCEL|wx.CENTRE)
      button = dlg.ShowModal()
      if button == wx.ID_OK:
         result = dlg.GetValue()
         dlg.Destroy()
         if not verify_variable_name(result):
            gui_util.message("Invalid variable name: " + result)
            return None
         else:
            return result
      else:
         dlg.Destroy()
         return None



import py.test

from gamera.core import *
init_gamera()

def test_mlcc_wrongcalls():
   img=Image((0,0),(8,8))
   img.draw_filled_rect((1,1),(3,3),2)
   img.set((1,5),3)
   py.test.raises(Exception, MlCc, img, 2, 1)
   py.test.raises(Exception, MlCc, img, "a", Rect((0,0),(8,8)))
   py.test.raises(Exception, MlCc, "abc", 2, Rect((0,0),(8,8)))
   py.test.raises(Exception, MlCc, [1,2])
   py.test.raises(Exception, MlCc, [img])

def test_mlcc():
   # create test image with different labels
   img=Image((0,0),(8,8))
   img.draw_filled_rect((1,1),(3,3),2)
   img.set((1,5),3)
   img.set((5,1),4)
   img.set((5,5),5)

   # test onebit plugins (highlight, features)
   rgb = img.to_rgb()
   mlcc1 = MlCc(img, 2, Rect((0,0),(8,8)))
   assert round(mlcc1.black_area()[0]) ==  9.0
   rgb.highlight(mlcc1, RGBPixel(255,0,0))
   assert RGBPixel(0,0,0) == rgb.get((5,5))
   mlcc1.add_label(5, Rect((0,0),(8,8)))
   assert round(mlcc1.black_area()[0]) == 10.0
   assert True == mlcc1.has_label(2)
   assert False == mlcc1.has_label(4)
   assert True == mlcc1.has_label(5)
   rgb.highlight(mlcc1,RGBPixel(0,255,0))
   assert RGBPixel(0,255,0) == rgb.get((5,5))

   # test conversion to Cc
   labels = mlcc1.get_labels()
   labels.sort()
   assert [2,5] == labels
   cc = mlcc1.convert_to_cc()
   assert 1 == len(mlcc1.get_labels())
   label = mlcc1.get_labels()[0]
   assert label == img.get((5,5))
   assert label == img.get((2,2))

   # test relabeling
   mlcc2 = MlCc(img, 2, Rect((1,1),(3,3)))
   mlcc2.add_label(3, Rect((1,5),(1,5)))
   mlcc2.add_label(4, Rect((5,1),(5,1)))
   mlcc2.add_label(5, Rect((5,5),(5,5)))
   assert [(1,1), (5,5)] == [mlcc2.ul, mlcc2.lr]
   mlcc3 = mlcc2.relabel([2,3])
   labels = mlcc3.get_labels(); labels.sort()
   assert [2,3] == labels
   assert [(1,1), (3,5)] == [mlcc3.ul, mlcc3.lr]
   mlcc3, mlcc4 = mlcc2.relabel([[2,3],[4]])
   labels = mlcc3.get_labels(); labels.sort()
   assert [2,3] == labels
   labels = mlcc4.get_labels(); labels.sort()
   assert [4] == labels
   assert [(1,1), (3,5)] == [mlcc3.ul, mlcc3.lr]
   assert [(5,1), (5,1)] == [mlcc4.ul, mlcc4.lr]
   mlcc2.remove_label(2)
   mlcc2.remove_label(3)
   labels = mlcc2.get_labels(); labels.sort()
   assert [4,5] == labels
   assert [(5,1),(5,5)] == [mlcc2.ul, mlcc2.lr]

   # test get/set
   mlcc2.set((0,1),3)
   assert 3 == img.get((5,2))
   assert 0 == mlcc2.get((0,1))
   mlcc2.set((0,1),5)
   assert 5 == img.get((5,2))
   assert 5 == mlcc2.get((0,1))
   cc = mlcc2.convert_to_cc()
   cc.set((0,2),20)
   assert 20 == img.get((5,3))
   assert 0 == cc.get((0,2))
   cc.set((0,2),cc.label)
   assert cc.label == img.get((5,3))
   assert cc.label == cc.get((0,2))

   # test constructor from Cc's
   img2 = img=Image((0,0),(8,8))
   img.draw_filled_rect((1,1),(3,3),1)
   img.set((5,5),1)
   ccs = img.cc_analysis()
   mlcc = MlCc(ccs)
   assert [(1,1),(5,5)] == [mlcc.ul, mlcc.lr]
   cclabels = [c.label for c in ccs]; cclabels.sort()
   mlcclabels = mlcc.get_labels(); mlcclabels.sort()
   assert cclabels == mlcclabels



#
#
# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
#               2010      Christoph Dalitz
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

from gamera.plugin import *

import _color

class ExtractFloatChannel(PluginFunction):
    self_type = ImageType([RGB])
    return_type = ImageType([FLOAT])
    doc_examples = [(RGB,)]

class ExtractGreyscaleChannel(PluginFunction):
    self_type = ImageType([RGB])
    return_type = ImageType([FLOAT])
    doc_examples = [(RGB,)]

class hue(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is a hue value in HSV space
    in range [0, 1).  Since the hue space is cyclic, the shortest
    *distance* between 1 and 0 is 0.
    """
    pass

class saturation(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is a saturation value in
    HSV space in range [0, 1].
    """
    pass

class value(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is the value in
    HSV space in range [0, 1].  For more information, Google for HSV color
    space.
    """
    pass

class cie_x(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is a *x* value in the `CIE
    1964 Colorimetric`__ system in range [0, 1).

    .. __: http://www.isc.tamu.edu/~astro/color/cie_xyz1964.html
    """
    pass

class cie_y(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is a *y* value in the `CIE
    1964 Colorimetric`__ system in range [0, 1).

    .. __: http://www.isc.tamu.edu/~astro/color/cie_xyz1964.html
    """
    pass

class cie_z(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is a *z* value in the `CIE
    1964 Colorimetric`__ system in range [0, 1).

    .. __: http://www.isc.tamu.edu/~astro/color/cie_xyz1964.html
    """
    pass

class cie_Lab_L(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is an *L* value in the 
    CIE L*a*b* color space. For an introduction to the different color
    spaces, see A. Ford and A. Roberts: `Color Space Concersions`__ (1998).

    The present conversion uses the RGB to Lab conversion routine from VIGRA.

    .. __: http://www.poynton.com/PDFs/coloureq.pdf
    """
    pass

class cie_Lab_a(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is an *a* value in the 
    CIE L*a*b* color space. For an introduction to the different color
    spaces, see A. Ford and A. Roberts: `Color Space Concersions`__ (1998).

    The present conversion uses the RGB to Lab conversion routine from VIGRA.

    .. __: http://www.poynton.com/PDFs/coloureq.pdf
    """
    pass

class cie_Lab_b(ExtractFloatChannel):
    """
    Returns a FLOAT image where each pixel is a *b* value in the 
    CIE L*a*b* color space. For an introduction to the different color
    spaces, see A. Ford and A. Roberts: `Color Space Concersions`__ (1998).

    The present conversion uses the RGB to Lab conversion routine from VIGRA.

    .. __: http://www.poynton.com/PDFs/coloureq.pdf
    """
    pass

class cyan(ExtractGreyscaleChannel):
    """
    Returns a GREYSCALE image where each pixel is the cyan component
    of the RGB original.
    """
    pass

class magenta(ExtractGreyscaleChannel):
    """
    Returns a GREYSCALE image where each pixel is the magenta
    component of the RGB original.
    """
    pass

class yellow(ExtractGreyscaleChannel):
    """
    Returns a GREYSCALE image where each pixel is the yellow component
    of the RGB original.
    """
    pass

class red(ExtractGreyscaleChannel):
    """
    Returns a GREYSCALE image where each pixel is the red component of
    the RGB original.
    """
    pass

class green(ExtractGreyscaleChannel):
    """
    Returns a GREYSCALE image where each pixel is the green component
    of the original.
    """
    pass

class blue(ExtractGreyscaleChannel):
    """
    Returns a GREYSCALE image where each pixel is the blue component
    of the RGB original.
    """
    pass


class false_color(PluginFunction):
    """
    Returns a false color representation of the given image.
    This can help visualize greyscale images that are not *real*
    images but are representations of other kinds of data.

    The option *colormap* specifies how the values are converted:

     0: diverging colormap after Moreland with blue representing low, white representing mean, and red representing high values

     1: rainbow colormap with blue representing low, green representing mean, and red representing high values
 
    Note that float images are scaled to the range [0,1], which means
    that the highest value is always colored red and the lowest value blue.
    For greyscale image no range stretching is done, so if you want this,
    you must first convert the image to a float image.

    Reference: K. Moreland:
    `Diverging Color Maps for Scientific Visualization.`__
    5th International Symposium on Visual Computing, 2009

    __ http://www.kennethmoreland.com/color-maps/
    """
    self_type = ImageType([FLOAT, GREYSCALE])
    args = Args([Choice("colormap", ["diverging", "rainbow"], default=0)])
    return_type = ImageType([RGB], "false_color")
    author = "Christoph Dalitz"
    doc_examples = [(GREYSCALE,0), (GREYSCALE,1)]
    def __call__(self, colormap=0):
        return _color.false_color(self, colormap)
    __call__ = staticmethod(__call__)

class colors_to_labels(PluginFunction):
    """
    Converts an RGB image to a labeled onebit image.

    Each RGB color is replaced by the label specified in the mapping
    *rgb_to_label*. RGB values not listed in *rgb_to_label* are white
    in the returned onebit image. When no mapping *rgb_to_label* is
    provided, each different RGB color is replaced by a unique label.

    This is mostly useful for reading manually labeled groundtruth
    data from color PNG files. Example:

    .. code:: Python

      # map red to label 3, and green to label 5
      labeled = rgb.colors_to_labels( {RGBPixel(255,0,0): 3, RGBPixel(0,255,0): 5} )

    A typical use case of this plugin is in combination
    with ccs_from_labeled_image_.

    .. _ccs_from_labeled_image: utility.html#ccs-from-labeled-image
    """
    self_type = ImageType([RGB])
    return_type = ImageType([ONEBIT])
    args = Args([Class("rgb_to_label", dict)])
    author = "Christoph Dalitz and Hasan Yildiz"
    def __call__(self, dict=None):
      return _color.colors_to_labels(self, dict)
    __call__ = staticmethod(__call__)


class ColorModule(PluginModule):
    category = "Color"
    cpp_headers = ["color.hpp"]
    functions = [hue, saturation, value, cyan, magenta, yellow,
                 cie_x, cie_y, cie_z, cie_Lab_L, cie_Lab_a, cie_Lab_b,
                 red, green, blue, false_color,
                 colors_to_labels]
    author = "Michael Droettboom and Karl MacMillan"
    url = "http://gamera.sourceforge.net/"

module = ColorModule()

del ExtractFloatChannel
del ExtractGreyscaleChannel



#
# Copyright (C) 2012 Tobias Bolten
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import wx
import time
import threading

try:
    from agw import pycollapsiblepane as PCP
except ImportError:
    try:
        import wx.lib.agw.pycollapsiblepane as PCP
    except ImportError:
        raise RuntimeError("Biollante requires at least wxPython 2.8.11")
        

from gamera.gui.gaoptimizer.ExpertSettingPanel import *
from gamera.gui.gaoptimizer.SelectionPanel import *
from gamera.gui.gaoptimizer.CrossoverPanel import *
from gamera.gui.gaoptimizer.MutationPanel import *
from gamera.gui.gaoptimizer.ReplacementPanel import *
from gamera.gui.gaoptimizer.StopCriteriaPanel import *
from gamera.gui.gaoptimizer.ParallelizationPanel import *

from gamera.gui import gui_util
from gamera import knnga

#-------------------------------------------------------------------------------
class GAWorker(threading.Thread):
#-------------------------------------------------------------------------------
    #---------------------------------------------------------------------------
    def __init__(self, GAOptimizer, frame):
    #---------------------------------------------------------------------------
        threading.Thread.__init__(self)
        self.GAOptimizer = GAOptimizer
        self.frame = frame

    #---------------------------------------------------------------------------
    def run(self):
    #---------------------------------------------------------------------------
        self.GAOptimizer.startCalculation()
        self.frame.timer.Stop()

        wx.CallAfter(self.frame.UpdatePanels)
        self.frame.settingsPanel.stopButton.Disable()
        self.frame.statusPanel.stopButton.Disable()

#-------------------------------------------------------------------------------
class GAInitialLOOWorker(threading.Thread):
#-------------------------------------------------------------------------------
    #---------------------------------------------------------------------------
    def __init__(self, classifier, frame):
    #---------------------------------------------------------------------------
        threading.Thread.__init__(self)
        self.classifier = classifier
        self.frame = frame

    #---------------------------------------------------------------------------
    def run(self):
    #---------------------------------------------------------------------------
        initLOO = self.classifier.leave_one_out()
        self.frame.settingsPanel.initLOO = initLOO

#-------------------------------------------------------------------------------
class SettingsPanel(wx.ScrolledWindow):
#-------------------------------------------------------------------------------
    #---------------------------------------------------------------------------
    def __init__(self, parent, id, frame):
    #---------------------------------------------------------------------------
        wx.ScrolledWindow.__init__(self, parent, id)

        self.frame = frame
        self.workerThread = None
        self.initLOO = None
        
        topSizer = wx.BoxSizer(wx.VERTICAL)
        
        # mode box
        modeBox = wx.StaticBox(self, -1, "Mode of operation")
        modeBoxSizer = wx.StaticBoxSizer(modeBox, wx.HORIZONTAL)
        modeBoxGridSizer = wx.GridBagSizer(hgap=5, vgap=5)
        modeBoxSizer.Add(modeBoxGridSizer, 1, wx.EXPAND)

        self.selectingLabel = "Feature selection"
        self.weightingLabel = "Feature weighting"

        self.featureSelection = wx.RadioButton(self, -1, self.selectingLabel)
        self.featureSelection.SetValue(True)
        self.featureWeighting = wx.RadioButton(self, -1, self.weightingLabel)
        self.normalization = wx.CheckBox(self, -1, "Normalization")

        modeBoxGridSizer.Add(self.featureSelection, pos=(0,0), \
            flag=wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, border=10)
        modeBoxGridSizer.Add(self.normalization, pos=(0,1), \
            flag=wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, border=10)
        modeBoxGridSizer.Add(self.featureWeighting, pos=(1,0), \
            flag=wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, border=10)

        self.Bind(wx.EVT_RADIOBUTTON, self.OnOpModeChange, self.featureSelection)
        self.Bind(wx.EVT_RADIOBUTTON, self.OnOpModeChange, self.featureWeighting)

        # basic settings box
        settingsBox = wx.StaticBox(self, -1, "Basic GA settings")
        settingsBoxSizer = wx.StaticBoxSizer(settingsBox, wx.VERTICAL)
        settingsBoxFlexSizer = wx.FlexGridSizer(0, 2, 5, 15)
        settingsBoxFlexSizer.AddGrowableCol(1)

        text = wx.StaticText(self, -1, "Population size:")
        settingsBoxFlexSizer.Add(text, 0, wx.ALIGN_CENTER_VERTICAL |wx.LEFT | wx.RIGHT | wx.TOP, 0)
        self.popSize = wx.SpinCtrl(self, -1, size=(100,-1), min=2, max=1000, value='75')
        settingsBoxFlexSizer.Add(self.popSize, 0, wx.ALIGN_LEFT | wx.LEFT | wx.RIGHT | wx.TOP, 0)

        text = wx.StaticText(self, -1, "Crossover rate:")
        settingsBoxFlexSizer.Add(text, 0, wx.ALIGN_BOTTOM | wx.LEFT | wx.RIGHT | wx.TOP, 0)
        self.crossoverRate = wx.Slider(self, -1, 95, 0, 100, style= wx.SL_LABELS)
        settingsBoxFlexSizer.Add(self.crossoverRate, 0,  wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 0)

        text = wx.StaticText(self, -1, "Mutation rate:")
        settingsBoxFlexSizer.Add(text, 0, wx.ALIGN_BOTTOM | wx.LEFT | wx.RIGHT | wx.TOP, 0)
        self.mutationRate = wx.Slider(self, -1, 5, 0, 100, style= wx.SL_LABELS)
        settingsBoxFlexSizer.Add(self.mutationRate, 0,  wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 0)

        settingsBoxSizer.Add(settingsBoxFlexSizer, 0, wx.ALL | wx.EXPAND, 10)

        # expert settings box
        expertPane = PCP.PyCollapsiblePane(self, -1, "Expert GA settings", \
            agwStyle=wx.CP_USE_STATICBOX)
        self.Bind(wx.EVT_COLLAPSIBLEPANE_CHANGED, self.OnPaneChanged, expertPane)
        self.MakeExpertPane(expertPane.GetPane())

        # buttons
        buttonSizer = wx.BoxSizer(wx.HORIZONTAL)

        self.startButton = wx.Button(self, -1, "Start")
        self.startButton.Disable()
        buttonSizer.Add(self.startButton, 0, wx.ALL | wx.EXPAND, 10)
        self.Bind(wx.EVT_BUTTON, self.OnButton, self.startButton)

        self.stopButton = wx.Button(self, -1, "Stop")
        self.stopButton.Disable()
        buttonSizer.Add(self.stopButton, 0, wx.ALL | wx.EXPAND, 10)
        self.Bind(wx.EVT_BUTTON, self.OnButton, self.stopButton)


        topSizer.Add(modeBoxSizer, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 25)
        topSizer.Add(settingsBoxSizer, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 25)
        topSizer.Add(expertPane, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 25)
        topSizer.Add(buttonSizer, 0, wx.ALL | wx.CENTER, 25)
        self.SetSizer(topSizer)

        width, height = topSizer.GetMinSize()
        self.SetScrollbars(10, 10, width/10, height/10)

    #---------------------------------------------------------------------------
    def MakeExpertPane(self, pane):
    #---------------------------------------------------------------------------
        sizer = wx.BoxSizer(wx.VERTICAL)

        notebook = wx.Notebook(pane, -1)

        self.selectionPanel = SelectionPanel(notebook, -1)
        self.crossoverPanel = CrossoverPanel(notebook, -1)
        self.mutationPanel = MutationPanel(notebook, -1)
        self.replacementPanel = ReplacementPanel(notebook, -1)
        self.stopCriteriaPanel = StopCriteriaPanel(notebook, -1)
        self.parallelizationPanel = ParallelizationPanel(notebook, -1)

        notebook.AddPage(self.selectionPanel, "Selection")
        notebook.AddPage(self.crossoverPanel, "Crossover")
        notebook.AddPage(self.mutationPanel, "Mutation")
        notebook.AddPage(self.replacementPanel, "Replacement")
        notebook.AddPage(self.stopCriteriaPanel, "Stop Criteria")
        from gamera.__compiletime_config__ import has_openmp
        if has_openmp:
            notebook.AddPage(self.parallelizationPanel, "Parallelization")

        sizer.Add(notebook, 1, wx.ALL | wx.EXPAND, 0)
        pane.SetSizer(sizer)

    #---------------------------------------------------------------------------
    def OnPaneChanged(self, event):
    #---------------------------------------------------------------------------
        # redo the layout ... necessary!
        self.Layout()
        
    #---------------------------------------------------------------------------
    def OnOpModeChange(self, event):
    #---------------------------------------------------------------------------
        opMode = event.GetEventObject().GetLabel()

        # crossover settings for selection and weighting
        # TODO: implement a more generic way to set the 'defaults'
        if opMode == self.selectingLabel:
            self.crossoverPanel.uniformCrossover.SetValue(True)
        if opMode == self.weightingLabel:
            for widget in self.crossoverPanel.genericWidgets:
                widget.SetValue(False)
                if widget in self.crossoverPanel.childWidgets:
                    for child in self.crossoverPanel.childWidgets[widget]:
                        child.Disable()


        panels = [self.crossoverPanel, self.mutationPanel]

        if opMode == self.selectingLabel:
            for panel in panels:
                for selectionWidget in panel.selectionWidgets:
                    self.TriggerWidgetsInPanel(panel, selectionWidget, True)
                for weightingWidget in panel.weightingWidgets:
                    self.TriggerWidgetsInPanel(panel, weightingWidget, False)
        elif opMode == self.weightingLabel:
            for panel in panels:
                for selectionWidget in panel.selectionWidgets:
                    self.TriggerWidgetsInPanel(panel, selectionWidget, False)
                for weightingWidget in panel.weightingWidgets:
                    self.TriggerWidgetsInPanel(panel, weightingWidget, True)

    #---------------------------------------------------------------------------
    def TriggerWidgetsInPanel(self, panel, widget, state):
    #---------------------------------------------------------------------------
        widget.Enable(state)
        if widget.GetValue():
            for child in panel.childWidgets[widget]:
                child.Enable(state)

    #---------------------------------------------------------------------------
    def OnButton(self, event):
    #---------------------------------------------------------------------------
        eventObject = event.GetEventObject()
        eventObjectLabel = eventObject.GetLabel()

        if eventObjectLabel == "Start":
            self.startCalculation()
        elif eventObjectLabel == "Stop":
            self.stopCalculation()

    #---------------------------------------------------------------------------
    def startCalculation(self):
    #---------------------------------------------------------------------------

        if self.frame.classifier == None:
            gui_util.message("No classifier loaded")
            return

        # Base GA settings
        base = knnga.GABaseSetting()

        if self.featureSelection.GetValue():
            base.opMode = knnga.GA_SELECTION
        elif self.featureWeighting.GetValue():
            base.opMode = knnga.GA_WEIGHTING

        # Set the normalization state in the used classifier
        if self.classifier.get_normalization_state() != self.normalization.GetValue():
            self.classifier.set_normalization_state(self.normalization.GetValue())

        base.popSize = self.popSize.GetValue()
        base.crossRate = self.crossoverRate.GetValue() / 100.0
        base.mutRate = self.mutationRate.GetValue() / 100.0

        # calculate initial leave one out rate for status display
        self.initLOO = None
        initLOOThread = GAInitialLOOWorker(self.frame.classifier, self.frame)
        initLOOThread.setDaemon(1)
        initLOOThread.start()

        # Selection GA settings
        selection = knnga.GASelection()

        for expertWidget in self.selectionPanel.GetAllSettingWidgets():
            if expertWidget.IsEnabled() and expertWidget.GetValue():
                if expertWidget.GetName() == "rouletteWheel":
                    selection.setRoulettWheel()
                elif expertWidget.GetName() == "rouletteWheelScaled":
                    selection.setRoulettWheelScaled(\
                        self.selectionPanel.roulettWheelPreasure.GetValue())
                elif expertWidget.GetName() == "stochUniSampling":
                    selection.setStochUniSampling()
                elif expertWidget.GetName() == "rankSelection":
                    selection.setRankSelection(\
                        self.selectionPanel.rankSelectionPreasure.GetValue(),\
                        self.selectionPanel.rankSelectionExponent.GetValue())
                elif expertWidget.GetName() == "tournamentSelection":
                    selection.setTournamentSelection(\
                        self.selectionPanel.TournamentSelectionTsize.GetValue())
                elif expertWidget.GetName() == "randomSelection":
                    selection.setRandomSelection()
                else:
                    raise RuntimeError("Unknown selection method chosen")

        # Crossover GA settings
        cross = knnga.GACrossover()

        crossOpSet = False
        for expertWidget in self.crossoverPanel.GetAllSettingWidgets():
            if expertWidget.IsEnabled() and expertWidget.GetValue():
                if expertWidget.GetName() == "nPointCrossover":
                    cross.setNPointCrossover(self.crossoverPanel.nPointCrossoverN.GetValue())
                elif expertWidget.GetName() == "uniformCrossover":
                    cross.setUniformCrossover(self.crossoverPanel.uniformCrossoverPref.GetValue())
                elif expertWidget.GetName() == "SBXcrossover":
                    cross.setSBXcrossover(self.frame.classifier.num_features, 0.0, 1.0, \
                        self.crossoverPanel.sbxCrossoverEta.GetValue())
                elif expertWidget.GetName() == "segmentCrossover":
                    cross.setSegmentCrossover(self.frame.classifier.num_features, 0.0, 1.0, \
                        self.crossoverPanel.segmentCrossoverAlpha.GetValue())
                elif expertWidget.GetName() == "hypercubeCrossover":
                    cross.setHypercubeCrossover(self.frame.classifier.num_features, 0.0, 1.0, \
                        self.crossoverPanel.hypercubeCrossoverAlpha.GetValue())
                else:
                    raise RuntimeError("Unknown crossover method chosen")
                crossOpSet = True
        if not crossOpSet:
            raise RuntimeError("At least one crossover operator must be chosen")

        # Mutation GA settings
        muta = knnga.GAMutation()

        mutOpSet = False
        for expertWidget in self.mutationPanel.GetAllSettingWidgets():
            if expertWidget.IsEnabled() and expertWidget.GetValue():
                if expertWidget.GetName() == "shiftMutation":
                    muta.setShiftMutation()
                elif expertWidget.GetName() == "swapMuation":
                    muta.setSwapMutation()
                elif expertWidget.GetName() == "inversionMutation":
                    muta.setInversionMutation()
                elif expertWidget.GetName() == "binaryMutation":
                    muta.setBinaryMutation(self.mutationPanel.binaryMutationRate.GetValue(), False)
                elif expertWidget.GetName() == "gaussMutation":
                    muta.setGaussMutation(self.frame.classifier.num_features, 0.0, 1.0, \
                        self.mutationPanel.gaussMutationSigma.GetValue(), \
                        self.mutationPanel.gaussMutationPchance.GetValue())
                else:
                    raise RuntimeError("Unknown mutation method chosen")
                mutOpSet = True
        if not mutOpSet:
            raise RuntimeError("At least one mutation operator must be chosen")

        # Replacement GA settings
        replacement = knnga.GAReplacement()

        for expertWidget in self.replacementPanel.GetAllSettingWidgets():
            if expertWidget.IsEnabled() and expertWidget.GetValue():
                if expertWidget.GetName() == "generationalReplacement":
                    replacement.setGenerationalReplacement()
                elif expertWidget.GetName() == "SSGAworse":
                    replacement.setSSGAworse()
                elif expertWidget.GetName() == "SSGAdetTournament":
                    replacement.setSSGAdetTournament(self.replacementPanel.ssgaDetTourTsize.GetValue())
                else:
                    raise RuntimeError("Unknown replacement method chosen")

        # Stop Criteria GA settings
        stop = knnga.GAStopCriteria()

        stopCritSet = False
        for expertWidget in self.stopCriteriaPanel.GetAllSettingWidgets():
            if expertWidget.IsEnabled() and expertWidget.GetValue():
                if expertWidget.GetName() == "bestFitnessStop":
                    stop.setBestFitnessStop(1.0)
                elif expertWidget.GetName() == "maxGenerations":
                    stop.setMaxGenerations(self.stopCriteriaPanel.maxGenerationCount.GetValue())
                elif expertWidget.GetName() == "maxFitnessEvals":
                    stop.setMaxFitnessEvals(self.stopCriteriaPanel.maxFitnessEvalCount.GetValue())
                elif expertWidget.GetName() == "steadyStateStop":
                    stop.setSteadyStateStop(self.stopCriteriaPanel.steadyContinueMin.GetValue(), \
                        self.stopCriteriaPanel.steadyContinueNoChange.GetValue())
                else:
                    raise RuntimeError("Unknown stop criteria chosen")
                stopCritSet = True
        if not stopCritSet:
            raise RuntimeError("At least one stop criteria must be chosen")

        # Parallelization GA settings
        para = knnga.GAParallelization()

        para.mode = self.parallelizationPanel.parallelEnabled.GetValue()
        para.thredNum = self.parallelizationPanel.threadNum.GetValue()

        # encapsulate all settings to the final algorithm
        ga = knnga.GAOptimization(self.frame.classifier, base, selection, cross, \
                                  muta, replacement, stop, para)

        self.workerThread = GAWorker(ga, self.frame)
        self.workerThread.setDaemon(1)

        self.frame.statusPanel.starttime = time.time()
        self.frame.timer.Start(1000)
        self.workerThread.start()

        self.frame.UpdatePanels()
        self.stopButton.Enable()
        self.frame.statusPanel.stopButton.Enable()

        # switch to status panel
        self.frame.notebook.SetSelection(1)

    #---------------------------------------------------------------------------
    def stopCalculation(self):
    #---------------------------------------------------------------------------
        if self.workerThread == None:
            return True

        if self.workerThread.GAOptimizer.status:
            response = wx.MessageDialog(self, "Are you sure you want to stop the "\
                "optimization? This may take a long time.\n(The current generation"\
                " will be finished)", "Stop?", wx.YES_NO).ShowModal()

            if response == wx.ID_NO:
                return False

        # TODO: thread for displaying status indicator?
        # ... gui still responsible?
        self.workerThread.GAOptimizer.stopCalculation()
        self.workerThread.join()
        self.workerThread = None


        self.frame.timer.Stop()
        self.frame.UpdatePanels()
        self.stopButton.Disable()
        self.frame.statusPanel.stopButton.Disable()

        return True



#
# Copyright (C) 2007-2009 Christoph Dalitz, Stefan Ruloff, Robert Butz,
#                         Maria Elhachimi, Ilya Stoyanov, Rene Baston
#               2010-2014 Christoph Dalitz
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

from gamera.plugin import *
from gamera.plugins.listutilities import kernel_density, argmax
from math import sqrt

import _pagesegmentation


class projection_cutting(PluginFunction):
    """
    Segments a page with the *Iterative Projection Profile Cuttings*
    method.

    The image is split recursively in the horizontal and vertical
    direction by looking for 'gaps' in the projection profile.  A
    'gap' is a contiguous sequence of projection values smaller than
    *noise* pixels. The splitting is done for each gap wider or higher
    than given thresholds *Tx* or *Ty*. When no further split points
    are found, the recursion stops.

    Whether the resulting segments represent lines, columns or
    paragraphs depends on the values for *Tx* and *Ty*. The return
    value is a list of 'CCs' where each 'CC' represents a found
    segment. Note that the input image is changed such that each pixel
    is set to its CC label.

    *Tx*:
      minimum 'gap' width in the horizontal direction.  When set to
      zero, *Tx* is set to the median height of all connected
      component * 7, which might be a reasonable assumption for the
      gap width between adjacent text columns.

    *Ty*:
      minimum 'gap' width in the vertical direction.  When set to
      zero, *Ty* is set to half the median height of all connected
      component, which might be a reasonable assumption for the gap
      width between adjacent text lines.

    *noise*:
      maximum projection value still considered as belonging to a
      'gap'.

    *gap_treatment*:
      decides how to treat gaps when *noise* is non zero.
      When 0 ('cut'), gaps are cut in the middle and the noise pixels
      in the gap are assigned to the segments.
      When 1 ('ignore'), noise pixels within the gap are not assigned
      to a segment, in other words, they are ignored.
    """
    self_type = ImageType([ONEBIT])
    args = Args([Int('Tx', default = 0), Int('Ty', default = 0), \
		 Int('noise', default = 0), Choice('gap_treatment', ["cut", "ignore"], default=0)])
    return_type = ImageList("ccs")
    author = "Maria Elhachimi and Robert Butz"
    def __call__(image, Tx = 0, Ty = 0, noise = 0, gap_treatment = 0):
	    return _pagesegmentation.projection_cutting(image, Tx, Ty, noise, gap_treatment)
    __call__ = staticmethod(__call__)

class runlength_smearing(PluginFunction):
    """
    Segments a page with the *Run Length Smearing* algorithm.

    The algorithm converts white horizontal and vertical runs shorter
    than given thresholds *Cx* and *Cy* to black pixels (this is the
    so-called 'run length smearing').

    The intersection of both smeared images yields the page segments
    as black regions. As this typically still consists small white
    horizontal gaps, these gaps narrower than *Csm* are in a final
    step also filled out.

    The return value is a list of 'CCs' where each 'CC' represents a
    found segment. Note that the input image is changed such that each
    pixel is set to its CC label.

    Arguments:

    *Cx*:
      Minimal length of white runs in the rows. When set to *-1*, it
      is set to 20 times the median height of all connected
      components.

    *Cy*:
      Minimal length of white runs in the columns. When set to *-1*,
      it is set to 20 times the median height of all connected
      components.

    *Csm*:
      Minimal length of white runs row-wise in the almost final
      image. When set to *-1*, it is set to 3 times the median height
      of all connected components.
    """
    self_type = ImageType([ONEBIT])
    return_type = ImageList("ccs")
    args = Args([Int('Cx', default = -1), Int('Cy', default = -1), \
		 Int('Csm', default = -1)])
    author = "Christoph Dalitz and Iliya Stoyanov"
    def __call__(image, Cx = -1, Cy = -1, Csm = -1):
	return _pagesegmentation.runlength_smearing(image, Cx, Cy, Csm)
    __call__ = staticmethod(__call__)


class bbox_merging(PluginFunction):
    """
    Segments a page by extending and merging the bounding boxes of the
    connected components on the page.

    How much the segments are extended is controlled by the arguments
    *Ex* and *Ey*. Depending on their value, the returned segments
    can be lines or paragraphs or something else.

    The return value is a list of 'CCs' where each 'CC' represents a
    found segment. Note that the input image is changed such that each
    pixel is set to its segment label.

    Arguments:

    *Ex*:
      How much each CC is extended to the left and right before merging.
      When *-1*, it is set to twice the median width of all CCs.

    *Ey*:
      How much each CC is extended to the top and bottom before merging.
      When *-1*, it is set to the median height of all CCs.
      This will typically segment into paragraphs.

      If you want to segment into lines, set *Ey* to something small like
      one sixth of the median symbol height.

    *iterations*:
      After merging intersecting bounding boxes, it can happen that the
      enclosing bounding boxes of different segments still intersect.
      If you do not want this, set *iterations* > 1 (two will typically be
      sufficient). If you however only want actually intersecting bounding
      boxes to be merged, set *iterations* to one.
    """
    self_type = ImageType([ONEBIT])
    return_type = ImageList("ccs")
    args = Args([Int('Ex', default = -1), Int('Ey', default = -1), Int('iterations', default=2)])
    pure_python = True
    author = "Rene Baston, Karl MacMillan, and Christoph Dalitz"

    def __call__(self, Ex=-1, Ey=-1, iterations=2):
        # bbox with contained cc indices
        class Bbox:
            def __init__(self, allccs, indices):
                self.ccs = allccs
                self.indices = indices
                if len(indices) == 1:
                    self.rect = Rect(allccs[indices[0]])
                else:
                    self.rect = allccs[indices[0]].union_images([allccs[i] for i in indices])
            def extend(self, Ex, Ey, img):
                ul_y = max(0, self.rect.ul_y - Ey)
                ul_x = max(0, self.rect.ul_x - Ex)
                lr_y = min(img.lr_y, self.rect.lr_y + Ey)
                lr_x = min(img.lr_x, self.rect.lr_x + Ex)
                nrows = lr_y - ul_y + 1
                ncols = lr_x - ul_x + 1
                self.rect = Rect(Point(ul_x, ul_y), Dim(ncols, nrows))
            def merge(self, other):
                self.indices += other.indices
                self.rect.union(other.rect)
        # does one merging step
        def merge_boxes(bboxes):
            from gamera import graph
            bboxes.sort(lambda b1, b2: b1.rect.ul_y-b2.rect.ul_y)
            g = graph.Graph(graph.UNDIRECTED)
            # build graph where edge means overlap of two boxes
            for i in range(len(bboxes)):
                g.add_node(i)
            for i in range(len(bboxes)):
                for j in range(i+1, len(bboxes)):
                    if bboxes[j].rect.ul_y > bboxes[i].rect.lr_y:
                        break
                    if bboxes[i].rect.intersects(bboxes[j].rect):
                        if not g.has_edge(i,j):
                            g.add_edge(i,j)
            new_bboxes = []
            for sg in g.get_subgraph_roots():
                seg = [n() for n in g.BFS(sg)]
                bbox = bboxes[seg[0]]
                for i in range(1, len(seg)):
                    bbox.merge(bboxes[seg[i]])
                new_bboxes.append(bbox)
            return new_bboxes

        # the actual plugin
        from gamera.core import Dim, Rect, Point, Cc
        from gamera.plugins.listutilities import median

        page = self.image_copy()
        ccs = page.cc_analysis()

        # compute average CC size
        if Ex == -1:
            Ex = 2*median([c.ncols for c in ccs])
        if Ey == -1:
            Ey = median([c.nrows for c in ccs])

        # create merged segments
        bboxes = [Bbox(ccs, [i]) for i in range(len(ccs))]
        for bb in bboxes:
            bb.extend(Ex, Ey, page)
        for i in range(iterations):
            oldlen = len(bboxes)
            bboxes = merge_boxes(bboxes)
            if oldlen == len(bboxes):
                break
        seg_ccs = []
        for i,bbox in enumerate(bboxes):
            label = i+1
            ccs_of_segment = [ccs[j] for j in bbox.indices]
            for cc in ccs_of_segment:
                self.highlight(cc, label)
            seg_ccs.append(Cc(self, label, ccs_of_segment[0].union_rects(ccs_of_segment)))
        return seg_ccs

    __call__ = staticmethod(__call__)


class kise_block_extraction(PluginFunction):
    """
    Segments a page into blocks by Kise's method based on the area Voronoi diagram
    as described in

        K. Kise, A. Sato, M. Iwata: *Segmentation of Page Images Using the
        Area Voronoi Diagram.* Computer Vision and Image Understandig 70,
        pp. 370-382, 1998

    The return value is a list of 'CCs' where each 'CC' represents a
    found segment. Note that the input image is changed such that each
    pixel is set to its segment label.

    The algorithm first builds a CC neighborhood graph and then removes edges from
    this graph based upon the area ratio and distance between adjacent segments.
    The criterion is

        *d/Td1* <= 1  OR  *d/Td2* + *A/Ta* <= 1

    where *Td1* < *Td2* are the two largest peaks in the CC distance distribution and
    A is the area ratio of the adjacent CCs.

    Arguments:

    *Ta*:
      Area ratio in the criterion above.

    *fr*:
      Fraction for determining Td2. It is not chosen as the peak position, but
      larger at the position where the peak has fallen to a fraction *fr* of its
      height.
    """
    self_type = ImageType([ONEBIT])
    return_type = ImageList("ccs")
    args = Args([Float('Ta', default = 40.0), Float('fr', default = 0.34)])
    pure_python = True
    author = "Christoph Dalitz"

    def __call__(self, Ta=40.0, fr=0.34):
        # compute neighborship graph
        from gamera.plugins.geometry import delaunay_from_points
        ccs = self.cc_analysis()
        i = 0
        points = []
        labels = []
        labels2ccs = {}
        for cc in ccs:
            p = cc.contour_samplepoints(15,1)
            cc.points = p
            points += p
            labels += [cc.label] * len(p)
            labels2ccs[cc.label] = cc
        neighbors = delaunay_from_points(points, range(len(points)))

        # compute edge properties
        class Edge(object):
            def __init__(self,cc1,cc2,d2):
                self.cc1 = cc1; self.cc2 = cc2
                self.d = d2
                a = [cc1.black_area()[0], cc2.black_area()[0]]
                self.ar = max(a)/min(a)
        labelneighbors = {}
        for pair in neighbors:
            if (labels[pair[0]] < labels[pair[1]]):
                label1 = labels[pair[0]]; label2 = labels[pair[1]]
            else:
                label1 = labels[pair[1]]; label2 = labels[pair[0]]
            if label1 == label2:
                continue
            p1 = points[pair[0]]
            p2 = points[pair[1]]
            d2 = (p1.x-p2.x)**2 + (p1.y-p2.y)**2
            key = "%i;%i" % (label1,label2)
            if not labelneighbors.has_key(key):
                labelneighbors[key] = Edge(labels2ccs[label1], labels2ccs[label2], d2)
            else:
                e = labelneighbors[key]
                if e.d > d2:
                    e.d = d2
        for e in labelneighbors.itervalues():
            e.d = sqrt(e.d)

        # determine thresholds Td1 and Td2 from distance statistics
        distances = [e.d for e in labelneighbors.itervalues()]
        distances.sort()
        if len(distances) > 50:
            distances = distances[len(distances)/20:(len(distances)-len(distances)/20)]
        x = [float(i*max(distances))/512.0 for i in range(512)]
        density = kernel_density(distances, x, kernel=2)
        local_maxima_i = []
        local_maxima_d = []
        for i in range(1,len(density)-1):
            if density[i] > density[i-1] and density[i] > density[i+1]:
                local_maxima_i.append(i)
                local_maxima_d.append(density[i])
        m1 = argmax(local_maxima_d)
        i1 = local_maxima_i[m1]
        local_maxima_i = [local_maxima_i[i] for i in range(len(local_maxima_i)) if i != m1]
        local_maxima_d = [local_maxima_d[i] for i in range(len(local_maxima_d)) if i != m1]
        i2 = local_maxima_i[argmax(local_maxima_d)]
        if i2 < i1:
            tmp = i2
            i2 = i1
            i1 = tmp
        dmax = density[i2]
        i2 += 1
        while i2 < len(x) - 1:
            if density[i2] < fr*dmax:
                break
            i2 += 1
        Td1 = x[i1]
        Td2 = x[i2]
        #print "Td1 =", Td1, "Td2 =", Td2, "(plugin)"

        # build graph
        from gamera import graph
        g = graph.Graph(graph.UNDIRECTED)
        rgb = self.to_rgb()
        for e in labelneighbors.itervalues():
            if not g.has_node(e.cc1.label):
                g.add_node(e.cc1.label)
            if not g.has_node(e.cc2.label):
                g.add_node(e.cc2.label)
            if (e.d/Td1 <= 1.0) or (e.d/Td2 + e.ar/Ta <= 1):
                g.add_edge(e.cc1.label, e.cc2.label)
            else:
                pass
        
        # split graph into connected subgraphs
        from gamera.core import MlCc
        seglabels = []
        for i,sg in enumerate(g.get_subgraph_roots()):
            seg = [n() for n in g.BFS(sg)]
            seglabels.append(seg)
        segments = []
        for lbs in seglabels:
            mlcc = MlCc([labels2ccs[lb] for lb in lbs])
            segments.append(mlcc.convert_to_cc())

        return segments

    __call__ = staticmethod(__call__)


class sub_cc_analysis(PluginFunction):
    """
    Further subsegments the result of a page segmentation algorithm into
    groups of actual connected components.

    The result of a page segmentation plugin is a list of 'CCs' where
    each 'CC' does not represent a 'connected component', but a page
    segment (typically a line of text). In a practical OCR application
    you will however need the actual connected components (which
    should roughly correspond to the glyphs) in groups of lines. That
    is what this plugin is meant for.

    The input image must be an image that has been processed with a
    page segmentation plugin, i.e. all pixels in the image must be
    labeled with a segment label. The input parameter *cclist* is the
    list of segments returned by the page segmentation algorithm.

    The return value is a tuple with two entries:

    - a new image with all pixels labeled according to the new CCs
    - a list of ImageLists, each list containing all connected components
      belonging to the same page segments


    .. note:: The groups will be returned in the same order as given
          in *cclist*.  This means that you can sort the page segments
          by reading order before passing them to *sub_cc_analysis*.

          Note that the order of the returned CCs within each group is
          not well defined. Hence you will generally need to sort each
          subgroup by reading order.
    """
    self_type = ImageType([ONEBIT])
    return_type = Class("img_ccs", tuple)
    args = Args([ImageList('cclist')])
    author = "Stephan Ruloff and Christoph Dalitz"


class textline_reading_order(PluginFunction):
    """
    Sorts a list of Images (CCs) representing textlines by reading order and
    returns the sorted list. Incidentally, this will not only work on
    textlines, but also on paragraphs, but *not* on actual Connected 
    Components.

    The algorithm sorts all lines in topological order, based on
    the following criteria for the pairwise order of two lines:

    - line *a* comes before line *b* when *a* is totally to the left
      of *b* (order \"column before row\")

    - line *a* comes before *b* when both overlap horizontally and
      *a* is above *b* (order within a column)

    In the reference `\"High Performance Document Analysis\"`__
    by T.M. Breuel (Symposium on Document Image Understanding,
    USA, pp. 209-218, 2003),
    an additional constraint is made for the first criterion by demanding
    that no other segment may be between *a* and *b* that overlaps
    horizontally with both. This constraint for taking multi column
    headings that interrupt columns into account is replaced in this
    implementation with an a priori sort of all textlines by *y*-position.
    This results in a preference of rows over columns (in case of ambiguity)
    in the depth-first-search utilized in the topological sorting.

    .. __: http://iupr1.cs.uni-kl.de/~shared/publications/2003-breuel-sdiut-high-performance-doc-layout-analysis.pdf

    As this function is not an image method, but a free function, it
    is not automatically imported with all plugins and you must import
    it explicitly with

    .. code:: Python

      from gamera.plugins.pagesegmentation import textline_reading_order

    """
    self_type = None
    return_type = ImageList("orderedccs")
    args = Args([ImageList("lineccs")])
    pure_python = True
    author = "Christoph Dalitz"
    def __call__(lineccs):
        # utilities for Gamera's graph API
        from gamera import graph
        from gamera import graph_util
        class SegForGraph:
            def __init__(self,seg):
                self.segment = seg
                self.label = 0
        #
        # build directed graph of all lines
        #
        G = graph.Graph(graph.FLAG_DAG)
        seg_data = [SegForGraph(s) for s in lineccs]
        # sort by y-position for row over column preference in ambiguities
        seg_data.sort(lambda s,t: s.segment.offset_y - t.segment.offset_y)
        G.add_nodes(seg_data)
        for s in seg_data:
            for t in seg_data:
                if s.segment.offset_x <= t.segment.offset_x + t.segment.ncols and \
                        s.segment.offset_x + s.segment.ncols >= t.segment.offset_x:
                    if s.segment.offset_y < t.segment.offset_y:
                        G.add_edge(s,t)
                elif s.segment.offset_x < t.segment.offset_x:
                        G.add_edge(s,t)
        #
        # compute topoligical sorting by depth-first-search
        #
        segs_sorted = [] # topologically sorted list
        def dfs_visit(node):
            node.data.label = 1
            for nextnode in node.nodes:
                if nextnode.data.label == 0:
                    dfs_visit(nextnode)
            segs_sorted.append(node.data.segment)
        for node in G.get_nodes():
            if node.data.label == 0:
                dfs_visit(node)
        segs_sorted.reverse() # correct that we always appended to the back
        return segs_sorted

    __call__ = staticmethod(__call__)


class segmentation_error(PluginFunction):
    """Compares a ground truth segmentation *Gseg* with a segmentation *Sseg*
and returns error count numbers.

The input images must be given in such a way that each segment is
uniquely labeled, similar to the output of a page segmentation
algorithm like `runlength_smearing`_. For ground truth data, such a labeled
image can be obtained from an external color image with `colors_to_labels`_.

.. _`runlength_smearing`: #runlength-smearing
.. _`colors_to_labels`: color.html#colors-to-labels

The two segmentations are compared by building equivalence classes of
overlapping segments as described in

  M. Thulke, V. Margner, A. Dengel:
  *A general approach to quality evaluation of document
  segmentation results.*
  Lecture Notes in Computer Science 1655, pp. 43-57 (1999)

Each class is assigned an error type depending on how many ground truth
and test segments it contains. The return value is a tuple
(*n1,n2,n3,n4,n5,n6)* where each value is the total number of classes
with the corresponding error type:

+------+-----------------------+---------------+----------------------+
| Nr   | Ground truth segments | Test segments | Error type           |
+======+=======================+===============+======================+
| *n1* | 1                     | 1             | correct              |
+------+-----------------------+---------------+----------------------+
| *n2* | 1                     | 0             | missed segment       |
+------+-----------------------+---------------+----------------------+
| *n3* | 0                     | 1             | false positive       |
+------+-----------------------+---------------+----------------------+
| *n4* | 1                     | > 1           | split                |
+------+-----------------------+---------------+----------------------+
| *n5* | > 1                   | 1             | merge                |
+------+-----------------------+---------------+----------------------+
| *n6* | > 1                   | > 1           | splits and merges    |
+------+-----------------------+---------------+----------------------+

The total segmentation error can be computed from these numbers as
*1 - n1 / (n1 + n2 + n3 + n4 + n5 + n6)*. The individual numbers can
be of use to determine what exactly is wrong with the segmentation.

As this function is not an image method, but a free function, it
is not automatically imported with all plugins and you must import
it explicitly with

.. code:: Python

      from gamera.plugins.pagesegmentation import segmentation_error
"""
    self_type = None
    args = Args([ImageType([ONEBIT], 'Gseg'), \
                 ImageType([ONEBIT], 'Sseg')])
    return_type = IntVector("errors", length=6)
    author = "Christoph Dalitz"


# module declaration
class PageSegmentationModule(PluginModule):
    cpp_headers = ["pagesegmentation.hpp"]
    cpp_namespace = ["Gamera"]
    category = "PageSegmentation"
    functions = [projection_cutting, runlength_smearing, bbox_merging, \
                     kise_block_extraction, sub_cc_analysis, textline_reading_order, \
                     segmentation_error]
module = PageSegmentationModule() # create an instance of the module

# free function instances
textline_reading_order = textline_reading_order()
segmentation_error = segmentation_error()



#!/usr/bin/env python
import sys, gc, os
from gamera.core import *
init_gamera()
import gamera.graph
import gamera.graph_util

#gc.set_debug(gc.DEBUG_LEAK)  
flags = [
      gamera.graph.DIRECTED, 
      gamera.graph.CYCLIC, #undirected
      gamera.graph.BLOB, #not tree
      gamera.graph.MULTI_CONNECTED,
      gamera.graph.SELF_CONNECTED,
      gamera.graph.FREE, #all flags set
      gamera.graph.TREE, #undirected, acyclic, single-connected, not-self-connected
      gamera.graph.FLAG_DAG, #directed, acyclic
      gamera.graph.UNDIRECTED #undirected, cyclic
   ]



# -----------------------------------------------------------------------------
# tests for small graphs only
# -----------------------------------------------------------------------------

# ------------------------------------------------------------------------------
def _test_flags(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   correct_flags = {
      gamera.graph.DIRECTED: [gamera.graph.DIRECTED], 
      gamera.graph.CYCLIC: [gamera.graph.CYCLIC], #undirected
      gamera.graph.BLOB: [gamera.graph.BLOB], #not tree
      gamera.graph.MULTI_CONNECTED: [],
      gamera.graph.SELF_CONNECTED: [],
      gamera.graph.FREE: [gamera.graph.DIRECTED, gamera.graph.CYCLIC, 
                          gamera.graph.BLOB, gamera.graph.MULTI_CONNECTED, 
                          gamera.graph.SELF_CONNECTED], #all flags set
      gamera.graph.TREE: [gamera.graph.TREE], 
         #undirected, acyclic, single-connected, not-self-connected
      gamera.graph.FLAG_DAG: [gamera.graph.DIRECTED, gamera.graph.BLOB, 
                              gamera.graph.FLAG_DAG], #directed, acyclic
      gamera.graph.UNDIRECTED: [gamera.graph.BLOB, gamera.graph.CYCLIC, 
                                gamera.graph.UNDIRECTED] #undirected, cyclic
   }

   if flag in [gamera.graph.DIRECTED, gamera.graph.FLAG_DAG, gamera.graph.FREE]:
      assert g.is_directed()
   else:
      assert g.is_undirected()

   if correct_flags.has_key(flag):# and f not in ignore_flags:
      for f in correct_flags[flag] :
         assert g.has_flag(f)



# ------------------------------------------------------------------------------
def _test_remove_difference(flag = gamera.graph.FREE):
   result = {
      gamera.graph.FREE:      [2,3,1,2,2,3,0,2],
      gamera.graph.DIRECTED:  [2,3,1,2,2,3,0,2],
      gamera.graph.CYCLIC:    [2,3,1,2,2,3,0,2], #undirected
      gamera.graph.BLOB:      [2,3,1,2,2,3,0,2],#not tree
      gamera.graph.MULTI_CONNECTED: [2,3,1,2,2,3,0,2],
      gamera.graph.SELF_CONNECTED:  [2,3,1,2,2,3,0,2],
      gamera.graph.FREE:      [2,3,1,2,2,3,0,2], #all flags set
      gamera.graph.TREE:      [2,3,1,2,2,3,0,2], 
         #undirected, acyclic, single-connected, not-self-conn
      gamera.graph.FLAG_DAG:  [2,3,1,2,2,3,0,2], #directed, acyclic
      gamera.graph.UNDIRECTED:      [2,3,1,2,2,3,0,2]#undirected, cyclic
   }

   g = gamera.graph.Graph()
   g.add_edge(1,2)
   g.add_edge(2,3)
   assert g.nedges == result[flag][0]
   assert g.nnodes == result[flag][1]
   g.remove_node(2)
   assert g.nedges == result[flag][2]
   assert g.nnodes == result[flag][3]
   assert "%s" % list(g.get_edges()) == "[<Edge from 1 to 3 (2.0)>]"
   del g
   
   g = gamera.graph.Graph()
   g.add_edge(1,2)
   g.add_edge(2,3)
   assert g.nedges == result[flag][4]
   assert g.nnodes == result[flag][5]
   g.remove_node_and_edges(2)
   assert g.nedges == result[flag][6]
   assert g.nnodes == result[flag][7]

   del g
 


# ------------------------------------------------------------------------------
def _test_colorize(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)

   for i in range(7):
      for j in range(7):
         g.add_edge(i,j)

   try:
      g.colorize(6)
      assert False
   except Exception:
      assert True

   del g


   g = gamera.graph.Graph(flag)
   for i in range(7):
      g.add_edge(i, i+1)

   g.colorize(6)

   colorcount = [0] * 6
   for i in range(7):
      colorcount[g.get_color(i)] += 1

   for i in range(6):
      assert colorcount[i] > 0 and colorcount[i] < 3

   del g



# ------------------------------------------------------------------------------
def _test_bfs(flag = gamera.graph.FREE):
   correct_bfs = {}
   a = [1,19,5,4,2,6,3,7]
   b = [1,5,2,6,3,7]
   correct_bfs = {
      gamera.graph.UNDIRECTED: a,
      gamera.graph.DIRECTED: b,
      gamera.graph.CYCLIC: a,
      gamera.graph.BLOB: a,
      gamera.graph.MULTI_CONNECTED: a,
      gamera.graph.SELF_CONNECTED: a,
      gamera.graph.FREE: b,
      gamera.graph.TREE: a,
      gamera.graph.FLAG_DAG: b
   }
   
   result_bfs = []
   g = gamera.graph.Graph(flag)
   g.add_edge(4,19)
   g.add_edge(19,1)
   g.add_edge(1,5)
   g.add_edge(5,2)
   g.add_edge(5,6)
   g.add_edge(2,6)
   g.add_edge(6,3)
   g.add_edge(2,3)
   g.add_edge(7,3)
   g.add_edge(3,7)

#   gamera.graph_util.graphviz_output(g, "graph_bfs_%d.viz" % flag)
   for n in g.BFS(1):
      result_bfs.append(n())

   assert correct_bfs[flag] == result_bfs

   del g



# ------------------------------------------------------------------------------
def _test_dfs(flag = gamera.graph.FREE):
   a = [1001,1005,1006,1003,1007,1002,10019,1004]
   b = [1001,1005,1006,1003,1007,1002]
   correct_dfs = { 
      gamera.graph.UNDIRECTED: a,
      gamera.graph.DIRECTED: b,
      gamera.graph.CYCLIC: a,
      gamera.graph.BLOB: a,
      gamera.graph.MULTI_CONNECTED: a,
      gamera.graph.SELF_CONNECTED: a,
      gamera.graph.FREE: b,
      gamera.graph.TREE: a,
      gamera.graph.FLAG_DAG: b,
   }
   result_dfs = []
   g = gamera.graph.Graph(flag)
   g.add_edge(1004,10019)
   g.add_edge(10019,1001)
   g.add_edge(1001,1005)
   g.add_edge(1005,1002)
   g.add_edge(1005,1006)
   g.add_edge(1002,1006)
   g.add_edge(1006,1003)
   g.add_edge(1002,1003)
   g.add_edge(1007,1003)
   g.add_edge(1003,1007)
   for n in g.DFS(1001):
      #print n
      result_dfs.append(n())
   if correct_dfs.has_key(flag):
      assert correct_dfs[flag] == result_dfs

   del g



# ------------------------------------------------------------------------------
def _test_dijkstra(flag = gamera.graph.FREE):
   #TODO more correct_paths
   g = gamera.graph.Graph(flag)
   correct_paths = {
      gamera.graph.FREE: {8: {8: (0.0, [8]), 1: (7.0, [1, 8]), 2: (15.0, [2, 7, 1, 8]), 6: (6.0, [6, 8]), 7: (12.0, [7, 1, 8])}, 1: {8: (7.0, [8, 7, 1]), 1: (0.0, [1]), 2: (8.0, [2, 7, 1]), 6: (9.0, [6, 2, 7, 1]), 7: (5.0, [7, 1])}, 2: {8: (4.0, [8, 7, 2]), 1: (11.0, [1, 8, 7, 2]), 2: (0.0, [2]), 6: (1.0, [6, 2]), 7: (2.0, [7, 2])}, 6: {8: (4.0, [8, 6]), 1: (11.0, [1, 8, 6]), 2: (19.0, [2, 7, 1, 8, 6]), 6: (0.0, [6]), 7: (16.0, [7, 1, 8, 6])}, 7: {8: (2.0, [8, 7]), 1: (9.0, [1, 8, 7]), 2: (3.0, [2, 7]), 6: (4.0, [6, 2, 7]), 7: (0.0, [7])}}

   }

   g.add_edges([
      (1,2,10,True), (1,7,5,True), (2,6,1,True), (2,7,2,True),
      (6,8,4,True), (7,8,2,True), (7,2,3,True), (7,6,9,True),
      (8,1,7,True), (8,6,6,True)
   ])

   assert g.nedges == 10
   p = g.dijkstra_shortest_path(2)
   print {flag: p}
   if correct_paths.has_key(flag):
      assert p == correct_paths[flag][2]
   del p
   del g
 


# ------------------------------------------------------------------------------
def _test_dijkstra_all_pairs(flag = gamera.graph.FREE):
   #TODO more correct_paths
   g = gamera.graph.Graph(flag)
   correct_paths = {
      gamera.graph.FREE: {8: {8: (0.0, [8]), 1: (7.0, [1, 8]), 2: (15.0, [2, 7, 1, 8]), 6: (6.0, [6, 8]), 7: (12.0, [7, 1, 8])}, 1: {8: (7.0, [8, 7, 1]), 1: (0.0, [1]), 2: (8.0, [2, 7, 1]), 6: (9.0, [6, 2, 7, 1]), 7: (5.0, [7, 1])}, 2: {8: (4.0, [8, 7, 2]), 1: (11.0, [1, 8, 7, 2]), 2: (0.0, [2]), 6: (1.0, [6, 2]), 7: (2.0, [7, 2])}, 6: {8: (4.0, [8, 6]), 1: (11.0, [1, 8, 6]), 2: (19.0, [2, 7, 1, 8, 6]), 6: (0.0, [6]), 7: (16.0, [7, 1, 8, 6])}, 7: {8: (2.0, [8, 7]), 1: (9.0, [1, 8, 7]), 2: (3.0, [2, 7]), 6: (4.0, [6, 2, 7]), 7: (0.0, [7])}}
   }

   g.add_edges([
      (1,2,10,True), (1,7,5,True), (2,6,1,True), (2,7,2,True),
      (6,8,4,True), (7,8,2,True), (7,2,3,True), (7,6,9,True),
      (8,1,7,True), (8,6,6,True)
   ])

   assert g.nedges == 10
   p = g.all_pairs_shortest_path()
   if correct_paths.has_key(flag):
      assert p == correct_paths[flag]
   del p

   del g



#------------------------------------------------------------------------------
def _test_subgraph_roots(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag);
   g.add_node(2)
   g.add_node(3)
      
   assert len(list(g.get_subgraph_roots())) == 2

   g.add_edges([(2,4), (5,7), (1,3), (8,9), (1,2), (5,6), (2,2)])
   g.add_node(10)
   tofind = [True, True, True, True]
   found = [False, False, False, False]

   i = 0
   if g.is_directed():
      for root in g.get_subgraph_roots():
         value = root()
         if(value == 1):
            found[0] = True
         elif(value == 5):
            found[1] = True
         elif(value == 8):
            found[2] = True
         elif(value == 10):
            found[3] = True
         i += 1
   else:
      for root in g.get_subgraph_roots():
         value = root()
         if(value <= 4):
            found[0] = True
         elif(value <= 7):
            found[1] = True
         elif(value <= 9):
            found[2] = True
         elif(value <=10):
            found[3] = True
         i += 1

   assert found == tofind
   assert i == 4


   del g



# ------------------------------------------------------------------------------
def _test_add_node(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   count_before = g.nnodes
   was_existing = g.has_node(43)
   assert g.add_node(43) == True
   assert (was_existing and g.nnodes == count_before) or \
         (not was_existing and g.nnodes == count_before + 1)

   assert g.add_node(43) == False

   del g



# ------------------------------------------------------------------------------
def _test_fully_connected(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)

   g.add_node(9)
   g.add_node(10)
   g.add_node(11)
   g.add_node(12)
   assert g.is_fully_connected() == False

   g.add_edge(9,10)
   g.add_edge(11,12)

   assert g.is_fully_connected() == False

   g.add_edge(10,11)
   print list(g.get_edges())
   print list(g.get_nodes())
   assert g.is_fully_connected() == True

   del g



# ------------------------------------------------------------------------------
def _test_is_cyclic(flag = gamera.graph.FREE): 
   g = gamera.graph.Graph(flag)
   assert g.is_cyclic() == False
   g.add_node(1)
   assert g.is_cyclic() == False

   g.add_edge(1,2)
   g.add_edge(2,3)

   assert g.is_cyclic() == False

   g.add_edge(3,1)
   assert g.is_cyclic() == True
   del g



# ------------------------------------------------------------------------------
def _test_is_multi_connected(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   g.add_nodes([1,2,3])
   g.add_edge(1,2)
   g.add_edge(1,2)
   assert g.is_multi_connected()
   del g

   g = gamera.graph.Graph(flag)
   if g.is_undirected():
      g.add_edge(1,2)
      g.add_edge(2,1)
      assert g.is_multi_connected()
      del g
      g = gamera.graph.Graph(flag)

   g.add_nodes([1,2,3])
   g.add_edge(1,2)
   assert not g.is_multi_connected()
   del g



# ------------------------------------------------------------------------------
def _test_is_self_connected(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   g.add_node([1,2,3])
   g.add_edge(1,2)
   assert not g.is_self_connected()
   g.add_edge(1,1)
   assert g.is_self_connected()
   del g



# ------------------------------------------------------------------------------
def _test_make_singly_connected(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   g.add_nodes([1,2,3,4,5,6,7,8])
   g.add_edges([(1,2),(1,2),(3,4),(4,5),(4,5),(5,6)])
   assert g.is_multi_connected()
   g.make_singly_connected()
   assert not g.is_multi_connected()
   assert g.nedges == 4
   del g



# ------------------------------------------------------------------------------
def _test_make_not_self_connected(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   g.add_nodes([1,2,3,4,5,6,7,8])
   g.add_edges([(1,1),(1,2),(2,3),(2,2),(3,4),(4,5),(5,5)])
   assert g.is_self_connected()
   g.make_not_self_connected()
   assert not g.is_self_connected()
   assert g.nedges == 4



# ------------------------------------------------------------------------------
def _test_make_acyclic(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   g.make_cyclic()
   g.add_edge(1,2,1,True)
   g.add_edge(2,3,1,True)
   g.add_edge(3,4,1,True)
   g.add_edge(4,5,1,True)
   g.add_edge(4,2,1,True)
   assert g.is_cyclic()
   assert g.nedges == 5
   g.make_acyclic()
   assert g.nedges == 4
   assert not g.is_cyclic()

   del g



# ------------------------------------------------------------------------------
def _test_make_tree(flag = gamera.graph.FREE):
   a = "[<Edge from 1 to 2 (10.0)>, <Edge from 1 to 7 (5.0)>, <Edge from 8 to 6 (6.0)>, <Edge from 1 to 8 (7.0)>]"
   b = "[<Edge from 1 to 2 (10.0)>, <Edge from 1 to 7 (5.0)>, <Edge from 7 to 6 (9.0)>, <Edge from 1 to 8 (7.0)>]"

   correct = { 
      gamera.graph.UNDIRECTED: a,
      gamera.graph.DIRECTED: b,
      gamera.graph.CYCLIC: a,
      gamera.graph.BLOB: a,
      gamera.graph.MULTI_CONNECTED: a,
      gamera.graph.SELF_CONNECTED: a,
      gamera.graph.FREE: b,
      gamera.graph.TREE: a,
      gamera.graph.FLAG_DAG: b,
   }

   g = gamera.graph.Graph(flag)
   g.add_edges([
      (1,2,10,True), (1,7,5,True), (2,6,1,True), (2,7,2,True),
      (6,8,4,True), (7,8,2,True), (7,2,3,True), (7,6,9,True),
      (8,1,7,True), (8,6,6,True)
   ])

   g.make_tree()

   if correct.has_key(flag):
      assert correct[flag]  == "%s" % list(g.get_edges())

   del g



# ------------------------------------------------------------------------------
def _test_check_insert_restrictions(flag = None):
   results = {
      gamera.graph.UNDIRECTED: [3, [(1,2),(2,3),(3,1)], []],
      gamera.graph.DIRECTED: [2, [(1,2),(2,3)], []],
      gamera.graph.CYCLIC: [3, [(1,2),(2,3),(3,1)], []],
      gamera.graph.BLOB: [3, [(1,2),(2,3)], []],
      gamera.graph.MULTI_CONNECTED: [None, None, []],
      gamera.graph.SELF_CONNECTED: [None, None, None],
      gamera.graph.FREE: [3, [(1,2),(2,3),(3,1)], [(3,3)]],
      gamera.graph.TREE: [2, [(1,2),(2,3)], []],
      gamera.graph.FLAG_DAG: [2, [(1,2),(2,3)], []]
   }

   
   g = gamera.graph.Graph(flag | gamera.graph.CHECK_ON_INSERT)
   g.add_nodes([1,2,3,4,5,6,7,8,9])
   
   #tests acyclic restriction
   g.add_edge(1,2)
   print g.nedges
   g.add_edge(2,3)
   print g.nedges
   g.add_edge(3,1)
   print g.nedges
   if results.has_key(flag) and results[flag][1] != None:
#      assert g.nedges == results[flag][0]
      result = [(e.from_node(), e.to_node()) for e in list(g.get_edges())]
      result.sort()
      correct = results[flag][1]
      correct.sort()
      assert result == correct

   del g


   #tests self-connected restriction
   g = gamera.graph.Graph(flag | gamera.graph.CHECK_ON_INSERT)
   g.add_nodes([1,2,3,4,5,6,7,8,9])
   g.add_edge(3,3)
   if results.has_key(flag) and results[flag][2] != None:
      result = [(e.from_node(), e.to_node()) for e in g.get_edges()]
      result.sort()
      correct = results[flag][2]
      correct.sort()
      assert result == correct


   #tests multi-connected restriction
   #TODO



# ------------------------------------------------------------------------------
def _test_minimum_spanning_tree(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   print g.add_edges([
      (9,10,4), (9,16,8), (10,16,11), (10,11,8), 
      (11,17,2), (11,14,4), (11,12,7), (12,13,9), 
      (12,14,14), (13,14,10), (14,15,2), (15,17,6), 
      (15,16,1), (16,17,7)]
   )
   assert g.nedges == 14

   try:
#      gamera.graph_util.graphviz_output(g, "mst_before_%d.viz" % flag)
      t = g.create_minimum_spanning_tree()

#      gamera.graph_util.graphviz_output(t, "mst_after_%d.viz" % flag)
      assert t.nnodes == 9
      assert t.nedges == 8
      assert "[<Edge from 15 to 16 (1.0)>, <Edge from 11 to 17 (2.0)>, <Edge from 14 to 15 (2.0)>, <Edge from 11 to 14 (4.0)>, <Edge from 9 to 10 (4.0)>, <Edge from 11 to 12 (7.0)>, <Edge from 9 to 16 (8.0)>, <Edge from 12 to 13 (9.0)>]" == str(list(t.get_edges()))

      del t

   except TypeError:
      assert g.is_directed()

   del g



# ------------------------------------------------------------------------------
def _test_spanning_tree(flag = gamera.graph.FREE):
   g = gamera.graph.Graph(flag)
   print g.add_edges([
      (9,10,4), (9,16,8), (10,16,11), (10,11,8), 
      (11,17,2), (11,14,4), (11,12,7), (12,13,9), 
      (12,14,14), (13,14,10), (14,15,2), (15,17,6), 
      (15,16,1), (16,17,7)]
   )
   assert g.nedges == 14

   bfsnodes = [n() for n in g.BFS(11)]
   bfsnodes.sort()

   t = g.create_spanning_tree(11)
   treenodes = [n() for n in t.get_nodes()]
   treenodes.sort()

   assert treenodes == bfsnodes

   del t
   del g





# -----------------------------------------------------------------------------
# tests for larger graphs
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
def _test_add_nodes(flag = gamera.graph.FREE, count = 2000):
   g = gamera.graph.Graph(flag)
   
   for i in range(1000,1000+count):
      assert g.has_node(i) == False

   for i in range(1000,1000+count):
      g.add_node(i)
      assert g.nnodes == i+1-1000
      assert g.has_node(i) == True
      assert g.get_node(i)() == i

   assert g.has_node(1000+count/2) == True
   assert g.get_node(1000+count/2)() == 1000+count/2

   for i in range(1000,1000+count):
      print i
      assert g.has_node(i) == True
      assert g.get_node(i)() == i

   try:
      g.get_node(1000 + 2*count)
      assert False
   except ValueError:
      pass

   del g



# ----------------------------------------------------------------------------
def _test_graph_copy(flag = gamera.graph.FREE, count = 1000):
   if count > 100:
      g = gamera.graph.Graph(flag)
      for i in range(0,count):
         g.add_edge(i, i+1)

      g.add_edge(count / 2, count / 2 + 20)
      assert g.nedges == count+1
      assert g.is_cyclic()
      h = g.copy(gamera.graph.CHECK_ON_INSERT)
      del g
      assert h.is_acyclic()
      assert h.nedges == count
      del h



# ------------------------------------------------------------------------------
def _test_remove_nodes(flag = gamera.graph.FREE, count = 250):
   g = gamera.graph.Graph(flag)
   g.add_nodes(range(0,count))
   assert g.nnodes == count
   for i in range(0, count):
      g.remove_node(i)
      assert g.nnodes == count - i - 1

   for i in range(0,count):
      assert not g.has_node(i)

   assert g.nnodes == 0

   del g



# ------------------------------------------------------------------------------
def _test_remove_node_and_edges(flag = gamera.graph.FREE, count = 250):
   g = gamera.graph.Graph(flag)

   for i in range(0,count):
      g.add_edge(i,i+1)

   g.add_edge(count, 0)
   assert g.nedges == count+1
   assert g.nnodes == count+1

   for i in range(0,count):
      g.remove_node_and_edges(i)
      assert g.nedges == count-i-1
   
   assert g.nnodes == 1 #only count
   assert g.nedges == 0

   assert [x() for x in g.get_nodes()] == [count]

   del g

   g = gamera.graph.Graph(flag)

   for i in range(0,count):
      g.add_edge(i,i+1)

   assert g.nedges == count
   assert g.nnodes == count+1

   for i in range(1,count):
      g.remove_node(i)
      assert g.nedges == count-i

   assert g.nnodes == 2
   print list(g.get_edges()) 
   assert g.nedges == 1
   g.remove_node(0)
   assert g.nedges == 0
   assert g.nnodes == 1 #only count

   assert [x() for x in g.get_nodes()] == [count]

   del g



# ------------------------------------------------------------------------------
def _test_remove_all_edges(flag = gamera.graph.FREE, count = 250):
   g = gamera.graph.Graph(flag)

   for i in range(0,count):
      g.add_edge(i,i+1)

   g.add_edge(count, 0)
   assert g.nedges == count+1
   assert g.nnodes == count+1

   g.remove_all_edges()
   assert g.nedges == 0
   assert g.nnodes == count+1

   del g



# ------------------------------------------------------------------------------
def _test_remove_node2(flag = gamera.graph.FREE, count = 250):
   g = gamera.graph.Graph()

   for i in range(0,count):
      g.add_edge(i,i+1)

   g.add_edge(count,0)
   assert g.nedges == count+1
   assert g.nnodes == count+1

   for i in range(0,count-1):
      g.remove_node(i)
      assert g.nedges == count-i

   g.remove_node(count-1)
   assert g.nedges == 0
   
   assert g.nnodes == 1 #only count
   assert g.nedges == 0

   assert [x() for x in g.get_nodes()] == [count]

   del g



# ------------------------------------------------------------------------------
def _test_add_nodes_sequence(flag = gamera.graph.FREE, count = 2000):
   g = gamera.graph.Graph(flag)
   for i in range(1000,1000+count):
      assert g.has_node(i) == False

   g.add_nodes(list(range(1000,1000+count)))

   assert g.has_node(1000+count/2) == True
   assert g.get_node(1000+count/2)() == 1000+count/2

   for i in range(1000,1000+count):
      assert g.has_node(i) == True
      assert g.get_node(i)() == i

   assert g.nnodes == count

   assert g.has_node(500) == False

   del g



# ------------------------------------------------------------------------------
def _test_has_edge(flag = gamera.graph.FREE, count = 250):
   g = gamera.graph.Graph(flag)
   for i in range(0,count):
      g.add_edge(i,i+1)

   for i in range(0,count):
      assert g.has_edge(i,i+1)
   
   for e in g.get_edges():
      assert g.has_edge(e)
      assert g.has_edge(e.from_node, e.to_node)
      assert g.has_edge(e.from_node(), e.to_node())

   assert g.nedges == count
   assert g.nnodes == count+1

   for i in range(0,count):
      g.remove_node(i)
   
   assert g.nnodes == 1 #only count
   assert g.nedges == 0

   assert [x() for x in g.get_nodes()] == [count]

   del g



# ------------------------------------------------------------------------------
def _test_remove_edge(flag = gamera.graph.FREE, count = 250):
   g = gamera.graph.Graph(flag)
   for i in range(0,count):
      g.add_edge(i,i+1)

   for i in range(0,count):
      assert g.has_edge(i,i+1)
   edgecount = g.nedges
   for e in list(g.get_edges()):
      from_node, to_node = e.from_node, e.to_node

      g.remove_edge(from_node, to_node)
      assert not g.has_edge(from_node(), to_node())
      edgecount -= 1
      assert edgecount == g.nedges

   assert g.nedges == 0
   assert g.nnodes == count+1
   assert [x() for x in g.get_nodes()] == range(0,count+1)
   
   try:
      g.remove_edge(5*count, 6*count)
      assert False
   except RuntimeError:
      pass

   del g

   g = gamera.graph.Graph()
   for i in range(0,count):
      g.add_edge(i,i+1)

   for i in range(0,count):
      assert g.has_edge(i,i+1)
    
   g.remove_all_edges()

   for i in range(0,count):
      assert not g.has_edge(i,i+1)

   print g.nedges
   print list(g.get_edges())
   assert g.nedges == 0
   assert g.nnodes == count+1
   assert [x() for x in g.get_nodes()] == range(0,count+1)

   del g



# ------------------------------------------------------------------------------
allocs = {"in_graph": 0, "add_edge_1": 0, "add_edge_2": 0, "check": 0}
deallocs = {"in_graph": 0, "add_edge_1": 0, "add_edge_2": 0, "check": 0}
objects = []
def _test_userdefined_class(flag = gamera.graph.FREE, count = 4096):
   class Test:
      def __init__(self, a, b="in_graph"):
         global objects
         global allocs
         objects.append(self)
         allocs[b] += 1
         self.a = a
         self.b = b

      def __cmp__(self, other):
         return cmp(self.a, other.a)

      def __eq__(self, other):
         return self.a == other.a

      def __lt__(self, other):
         return self.a < other.a
      
      def __repr__(self):
         return "Test(%s, %s)" % (self.a, self.b)

      def __del__(self):
#         global deallocs
#         deallocs[self.b] += 1
         pass

   g = gamera.graph.Graph(gamera.graph.UNDIRECTED)
   g.add_node(Test(0))
   for i in range(0, count):
      g.add_node(Test(i+1))
      g.add_edge(Test(i, "add_edge_1"),Test(i+1, "add_edge_2"))

   for node in g.get_nodes():
      o = node()
  
   for i in range(0, count):
      assert Test(i) == g.get_node(Test(i))()

   for e in g.get_edges():
      assert e.from_node().a == e.to_node().a-1
      print "Edge %s %s \n" % (e.from_node(), e.to_node())

   allocs = {"in_graph": 0, "add_edge_1": 0, "add_edge_2": 0, "check": 0}
   deallocs = {"in_graph": 0, "add_edge_1": 0, "add_edge_2": 0, "check": 0}
   objects = []

   del g
 


# ------------------------------------------------------------------------------
def _test_large_graph(flag = gamera.graph.FREE, count = 20000):
   gc.collect()
   print "memlarge: %s" % memory_usage()
   g = gamera.graph.Graph(flag)
   g.add_node(0)
   for i in range(0, count):
      g.add_node(i+1)
      g.add_edge(i,i+1)

   for i in range(0, count):
      assert g.has_node(i) == True
      assert g.has_edge(i,i+1) == True
      assert g.get_node(i)() == i

   assert g.nnodes == count + 1
   g.remove_edge(count/2, count/2+1)
   del g
   gc.collect()
   print "memlarge_after: %s" % memory_usage()







# ------------------------------------------------------------------------------
def memory_usage():
      cmd = "ps u -p %i | awk '{sum=sum+$6}; END {print sum}'" % os.getpid()
      p = os.popen(cmd, "r")
      memory = p.readline().strip()
      p.close()
      return memory



# ------------------------------------------------------------------------------
def refcount():
      count = 0
      print len(gc.get_objects())
      for obj in gc.get_objects():
         count += sys.getrefcount(obj)
   
      return count



# ------------------------------------------------------------------------------
def _test_memory(flag = gamera.graph.FREE, count = 100):


   new_usage = None
   old_usage = new_usage
   for i in range(0,5):
      old_usage=new_usage
      _test_large_graph(flag)
      gc.collect()
      new_usage = (memory_usage(), refcount())
      print old_usage, new_usage
      assert old_usage == new_usage or old_usage == None or i < 2

   print "memory: %s | refcount: %d" % (memory_usage(), refcount())
   new_usage = None
   for i in range(0,count):
      old_usage = new_usage
      g = gamera.graph.Graph(flag)
      g.add_node(200003+i)
      g.add_node(300003+i)
      g.add_edge(200003+i,300003+i)
      b = g.get_node(200003+i)
      assert g.has_node(200003+i) == True
      assert g.has_edge(200003+i, 300003+i)
      del g
      g = None
      gc.collect()
      new_usage = (memory_usage(), refcount())
      print old_usage,new_usage
      assert old_usage == new_usage or old_usage == None or i < 2



tests_small = [
   _test_flags,
   _test_remove_difference,
   _test_bfs,
   _test_dfs,
   _test_dijkstra,
   _test_dijkstra_all_pairs,
   _test_subgraph_roots,
   _test_add_node,
   _test_fully_connected,
   _test_is_cyclic,
   _test_is_multi_connected,
   _test_is_self_connected,
   _test_make_singly_connected,
   _test_make_not_self_connected,
   _test_make_acyclic,
   _test_make_tree,
   _test_check_insert_restrictions,
   _test_minimum_spanning_tree,
   _test_spanning_tree,
   _test_colorize
]

tests_large = [
   _test_add_nodes,
   _test_graph_copy,
   _test_remove_nodes,
   _test_remove_node_and_edges,
   _test_remove_node2,
   _test_remove_all_edges,
   _test_add_nodes_sequence,
   _test_has_edge,
   _test_remove_edge,
   _test_userdefined_class,
   _test_large_graph
]

#   _test_memory


#tests = [_test_dijkstra]

# ------------------------------------------------------------------------------
def _test_memory_all():
   for test in tests:#lag in flags:
      for flag in flags:#test in tests:
#      try:
         print >>sys.stderr, flag, test
         new_usage = None
         for i in range(0,10):
            old_usage = new_usage
            test(flag)
            gc.collect()
            new_usage = (memory_usage(), refcount())
#      print >>sys.stderr, (old_usage, new_usage)
            if old_usage != None:
               if not ( (old_usage[1] == new_usage[1] \
                         and (int(new_usage[0])-int(old_usage[0]) < 2  )) \
                       or i < 4):
                  print >>sys.stderr, (old_usage, new_usage)

'''      except AssertionError:
         print >>sys.stderr, "%s failed !!!" %test
         info = sys.exc_info()
         exc_type = info[0]
         exc_value = info[1]
         exc_traceback = info[2]
                     
         trace = traceback.extract_tb(sys.exc_traceback)
         print >>sys.stderr, "Exception Type:  ", exc_type
         print >>sys.stderr, "Error Message:   ", exc_value
         print >>sys.stderr, "File name:       ", trace[0][0]
         print >>sys.stderr, "Error message:   ", trace[0][1]
         print >>sys.stderr, "Line:            ", trace[0][2]
         print >>sys.stderr, "Function:        ", trace[0][3]
'''



# ------------------------------------------------------------------------------
#prepare py.test
class TestGraph:
   pass

def _make_test(t, flag):
   def _t(self):
      return t(flag)
   return _t

def _make_test_size(t, flag, size):
   def _t(self):
      return t(flag, size)
   return _t

#small tests
for test in tests_small:
   for flag in flags:
      setattr(TestGraph, "test_small_%d_%s" % (flag, test.__name__ ), _make_test(test,flag))

#large default
#for test in tests_large:
#   for flag in flags:
#      setattr(TestGraph, "test_large_default_%d_%s" % (flag, test.__name__ ), _make_test(test,flag))

#large non-default
for size in [512]:
   for test in tests_large:
      for flag in flags:
         setattr(TestGraph, "test_large_%d_%d_%s" % (flag, size, test.__name__ ), 
                 _make_test_size(test, flag, size))
   




if __name__ == "__main__":
   t = TestGraph()
   for m in dir(t):
      if m.startswith("test_"):
         print m
         test = getattr(t, m)
         print test
         try:
            test()
         except AssertionError:
            pass
         #est(TestGraph)

   _test_memory_all()





# ------------------------------------------------------------------------------
def test_textline_reading_order():
   from gamera.plugins.pagesegmentation import textline_reading_order
   correct_orders = {"data/reading_order_2.png": [(42, 40, 1462, 114), (42, 158, 683, 232), (42, 276, 683, 350), (42, 418, 683, 492), (42, 560, 683, 633), (822, 158, 1462, 633), (42, 701, 1462, 775), (42, 843, 494, 917), (562, 843, 1132, 917), (42, 985, 683, 1059), (822, 985, 1132, 1059), (1200, 843, 1462, 1059)],
                     "data/reading_order.png": [(51, 56, 1471, 130), (51, 174, 691, 248), (51, 292, 691, 366), (51, 434, 691, 508), (51, 576, 691, 649), (830, 174, 1471, 508), (830, 576, 1471, 649), (51, 717, 1471, 791), (51, 859, 691, 933), (51, 1001, 691, 1075), (830, 859, 1471, 933), (830, 1001, 1471, 1075)]
                    }

   for file, correct in correct_orders.items():
      img = load_image(file)
      ccs = img.cc_analysis()
      ro = textline_reading_order(ccs)

      result = [(a.ul_x, a.ul_y, a.lr_x, a.lr_y) for a in ro]
      assert result == correct
      del ro
      del ccs
      del img





# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom,
#                          and Karl MacMillan
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

from __future__ import generators
from gamera import util
import sys, re

# GROUPING IN GENERAL:
#
# The grouping classifier interface allows glyphs to be grouped according to
# their physical relationships to each other.  The user classifies a set of
# glyphs as a "group", and other glyphs matching this general structure can
# later be located on the page.
#
# For documentation, there are two kinds of glyphs involved in this process:
#   part glyphs -- glyphs that should be grouped to form larger glyphs
#   union glyphs -- glyphs made up of part glyphs

class GridIndex:
   """Indexes glyphs using a grid, so glyphs near a given glyph are easier
   to find."""
   def __init__(self, glyphs, max_width=100, max_height=100):
      """Creates a grid index to store the given set of glyphs.  Note that
      the init function only creates a grid big enough to hold the glyphs,
      it does not actually store them...  That must be done by calling
      GridIndex.add_glyph.  max_width and max_height are the maximum size
      (in pixels) of each cell."""
      glyphs = util.make_sequence(glyphs)
      if len(glyphs) == 0:
         raise ValueError(
             "GridIndex must be initialised with at least one glyph")
      self.grid_rect = glyphs[0].union_rects(glyphs)
      self.cell_width = int(max_width)
      self.cell_height = int(max_height)
      self.cell_ncols = int(self.grid_rect.width / self.cell_width) + 1
      self.cell_nrows = int(self.grid_rect.height / self.cell_height) + 1
      self._create_cells()

   def _create_cells(self):
      self.grid = []
      for i in range(self.cell_ncols * self.cell_nrows):
         self.grid.append([])

   def add_glyph(self, glyph):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      self.grid[row * self.cell_ncols + col].append(glyph)

   def get_cell(self, row, col):
      if row < 0 or row >= self.cell_nrows:
         return []
      if col < 0 or col >= self.cell_ncols:
         return []
      return self.grid[row * self.cell_ncols + col]

   def get_cell_at_glyph(self, glyph):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      return self.get_cell(row, col)

   def get_glyphs_around_glyph(self, glyph):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      for r, c in self.search_order:
         ri = r + row
         ci = c + col
         cell_nrows = self.cell_nrows
         cell_ncols = self.cell_ncols
         if ri < 0 or ri >= cell_nrows or ci < 0 or ci >= cell_ncols:
            continue
         for glyph in self.grid[ri * cell_ncols + ci]:
            yield glyph
   search_order = ((0,0),                          # center
                   (-1,0), (0,-1), (1,0), (0,1),   # +
                   (-1,-1), (-1,1), (1,-1), (1,1)) # x

class GridIndexWithKeys(GridIndex):
   """Extends the basic GridIndex class to allow glyphs to also be stored and
   retreived by an arbitrary key."""

   def _create_cells(self):
      self.flat = {}
      self.grid = []
      for i in range(self.cell_ncols * self.cell_nrows):
         self.grid.append({})

   def add_glyph_by_key(self, glyph, key):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      cell_index = row * self.cell_ncols + col
      self.grid[cell_index].setdefault(key, []).append(glyph)
      self.flat.setdefault(key, []).append(glyph)

   def get_cell_by_key(self, row, col, key):
      if row < 0 or row >= self.cell_nrows:
         return []
      if col < 0 or col >= self.cell_ncols:
         return []
      return self.grid[row * self.cell_ncols + col].get(key, [])

   def get_cell_at_glyph_by_key(self, glyph, key):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      return self.grid[row * self.cell_ncols + col].get(key, [])

   def get_glyphs_around_glyph(self, glyph):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      cell_nrows = self.cell_nrows
      cell_ncols = self.cell_ncols
      for r, c in self.search_order:
         ri = r + row
         ci = c + col
         if ri < 0 or ri >= cell_nrows or ci < 0 or ci >= cell_ncols:
            continue
         for mapping in self.grid[ri * cell_ncols + ci].values():
            for glyph in mapping:
               yield glyph

   def get_glyphs_around_glyph_by_key(self, glyph, key):
      if not self.grid_rect.contains_point(glyph.center):
         raise ValueError(
             "glyph is not within the bounding box of the initial set of images")
      row = (glyph.center_y - self.grid_rect.ul_y) / self.cell_height
      col = (glyph.center_x - self.grid_rect.ul_x) / self.cell_width
      cell_nrows = self.cell_nrows
      cell_ncols = self.cell_ncols
      for r, c in self.search_order:
         ri = r + row
         ci = c + col
         if ri < 0 or ri >= cell_nrows or ci < 0 or ci >= cell_ncols:
            continue
         for glyph in self.grid[ri * cell_ncols + ci].get(key, []):
            yield glyph

   def get_glyphs_by_key(self, key):
      return self.flat.get(key, [])



from gamera.gameracore import FloatPoint as FP
from math import sqrt

def test_floatpoint():
   p = FP(2, 3)
   assert p.x == 2.0 and p.y == 3.0

   assert FP(2, 2) + FP(1, 2) == FP(3, 4)
   assert FP(2, 2) * FP(2, 3) == FP(4, 6)
   assert FP(2, 2) - FP(2, 4) == FP(0, -2)
   assert FP(2, 2) / FP(2, 3) == FP(1, 2.0/3.0)

   assert FP(0, 0).distance(FP(-3, -3)) == sqrt(18)

   assert repr(FP(2, 3)) == "FloatPoint(2, 3)"

def test_floatpoint_leak():
   for i in xrange(1000):
      p = FP(i, i)



#
# Copyright (C) 2012 Tobias Bolten
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import wx
import time

#-------------------------------------------------------------------------------
class StatusPanel(wx.ScrolledWindow):
#-------------------------------------------------------------------------------
    #---------------------------------------------------------------------------
    def __init__(self, parent, id, frame):
    #---------------------------------------------------------------------------
        wx.ScrolledWindow.__init__(self, parent, id)

        self.frame = frame
        self.starttime = None

        topSizer = wx.BoxSizer(wx.VERTICAL)

        # status box
        statusBox = wx.StaticBox(self, -1, "Status GA optimization")
        statusBoxSizer = wx.StaticBoxSizer(statusBox, wx.HORIZONTAL)
        statusFlexGridSizer = wx.FlexGridSizer(0, 2, 5, 15)

        # status indicator
        text = wx.StaticText(self, -1, "Current Status:")
        statusFlexGridSizer.Add(text, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)
        self.status = wx.StaticText(self, -1, "not running")
        statusFlexGridSizer.Add(self.status, 0, wx.wx.LEFT | wx.TOP | wx.RIGHT, 10)

        # status indicator
        text = wx.StaticText(self, -1, "Initial leave one out rate:")
        statusFlexGridSizer.Add(text, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)
        self.initLOO = wx.StaticText(self, -1, "unknown")
        statusFlexGridSizer.Add(self.initLOO, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)

        # time counter
        text = wx.StaticText(self, -1, "Elapsed time:")
        statusFlexGridSizer.Add(text, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)
        self.time = wx.StaticText(self, -1, "00 h 00 m 00 s")
        statusFlexGridSizer.Add(self.time, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)

        # generation counter
        text = wx.StaticText(self, -1, "Current generation:")
        statusFlexGridSizer.Add(text, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)
        self.curGen = wx.StaticText(self, -1, "unknown")
        statusFlexGridSizer.Add(self.curGen, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)

        # current rate
        text = wx.StaticText(self, -1, "Best leave one out rate:")
        statusFlexGridSizer.Add(text, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)
        self.curLOO = wx.StaticText(self, -1, "unknown")
        statusFlexGridSizer.Add(self.curLOO, 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)

        statusBoxSizer.Add(statusFlexGridSizer, 1, wx.ALL | wx.EXPAND, 10)

        # stop button
        self.stopButton = wx.Button(self, -1, "Stop")
        self.stopButton.Disable()
        self.Bind(wx.EVT_BUTTON, self.frame.settingsPanel.OnButton, self.stopButton)
        statusBoxSizer.Add(self.stopButton, 1, wx.ALL | wx.CENTER, 10)

        # statistics box
        statsBox = wx.StaticBox(self, -1, "GA statistics")
        statsBoxSizer = wx.StaticBoxSizer(statsBox, wx.VERTICAL)

        self.statistics = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE | wx.TE_READONLY)
        statsBoxSizer.Add(self.statistics, 1, wx.ALL | wx.EXPAND, 10)


        topSizer.Add(statusBoxSizer, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 25)
        topSizer.Add(statsBoxSizer, 1, wx.ALL | wx.EXPAND, 25)
        self.SetSizer(topSizer)

        width, height = topSizer.GetMinSize()
        self.SetScrollbars(10, 10, width/10, height/10)

    #---------------------------------------------------------------------------
    def Update(self):
    #---------------------------------------------------------------------------
        # init rate
        if self.frame.settingsPanel.initLOO == None:
            self.initLOO.SetLabel("unknown")
        else:
            rate = self.frame.settingsPanel.initLOO
            rate = (rate[0] / float(rate[1])) * 100
            self.initLOO.SetLabel("%.2lf %%" % rate)

        # consumed time
        if self.starttime != None:
            now = time.time()
            timediff = now - self.starttime

            hours = int(timediff / 3600.0)
            minutes = int((timediff % 3600) / 60.0)
            secs = int((timediff % 60))

            self.time.SetLabel("%d h %d m %d s" % (hours, minutes, secs))

        # status, generation count, current rate and monitor
        if self.frame.settingsPanel.workerThread == None:
            self.status.SetLabel("not running")
        else:
            if self.frame.settingsPanel.workerThread.isAlive():
                self.status.SetLabel("running")
            else:
                self.status.SetLabel("not running")

            self.curGen.SetLabel("%d" % self.frame.settingsPanel.workerThread.GAOptimizer.generation)
            self.curLOO.SetLabel("%.2lf %%" % (self.frame.settingsPanel.workerThread.GAOptimizer.bestFitness * 100))

            wx.CallAfter(self.statistics.SetValue, self.frame.settingsPanel.workerThread.GAOptimizer.monitorString)
            wx.CallAfter(self.statistics.ShowPosition, self.statistics.GetLastPosition())



#
# Copyright (C) 2012 Tobias Bolten
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import wx

try:
    from agw import floatspin as FS
except ImportError:
    try:
        import wx.lib.agw.floatspin as FS
    except ImportError:
        raise RuntimeError("Biollante requires at least wxPython 2.8.11")

from gamera.gui.gaoptimizer.ExpertSettingPanel import *

#-------------------------------------------------------------------------------
class CrossoverPanel(ExpertSettingPanel):
#-------------------------------------------------------------------------------
    #---------------------------------------------------------------------------
    def __init__(self, parent, id):
    #---------------------------------------------------------------------------
        ExpertSettingPanel.__init__(self, parent, id)

        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        # ---------------- generic crossovers ----------------
        genericBox = wx.StaticBox(self, -1, "Generic operations", size=(500,-1))
        genericBoxSizer = wx.StaticBoxSizer(genericBox, wx.VERTICAL)
        genericFlexGridSizer = wx.FlexGridSizer(0, 2, 5, 5)
        genericBoxSizer.Add(genericFlexGridSizer, 0)
        sizer.Add(genericBoxSizer, 0, wx.ALL | wx.EXPAND, 10)

        # n point crossover (default: one point crossover)
        nPointSizer = wx.BoxSizer(wx.HORIZONTAL)
        genericBoxSizer.Add(nPointSizer, 0, wx.EXPAND)
        
        self.nPointCrossover = wx.CheckBox(self, -1, "N-Point crossover", \
            name = "nPointCrossover")
        genericFlexGridSizer.Add(self.nPointCrossover, 0, \
            wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 5)
        
        self.nPointCrossoverN = wx.SpinCtrl(self, -1, size=(100,-1), \
            min=1, max=10, value='1') # TODO: max = vector dimension
        self.nPointCrossoverN.Disable()
        self.nPointCrossoverN.SetToolTipString("num crossover points")
        genericFlexGridSizer.Add(self.nPointCrossoverN, 0, wx.LEFT | wx.TOP | wx.RIGHT, 5)

        self.genericWidgets.append(self.nPointCrossover)
        self.AddChildToParent(self.nPointCrossover, self.nPointCrossoverN)

        # uniform crossover
        self.uniformCrossover = wx.CheckBox(self, -1, "Uniform crossover", \
            name = "uniformCrossover")
        self.uniformCrossover.SetValue(True)
        genericFlexGridSizer.Add(self.uniformCrossover, 0, \
            wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 5)

        self.uniformCrossoverPref = FS.FloatSpin(self, -1, min_val=0.0, max_val=1.0, \
            increment=0.01, value=0.5)
        self.uniformCrossoverPref.SetFormat("%f")
        self.uniformCrossoverPref.SetDigits(2)
        self.uniformCrossoverPref.SetToolTipString("Preference")
        genericFlexGridSizer.Add(self.uniformCrossoverPref, 0, wx.LEFT | wx.TOP | wx.RIGHT, 5)

        self.genericWidgets.append(self.uniformCrossover)
        self.AddChildToParent(self.uniformCrossover, self.uniformCrossoverPref)

        # ---------------- real crossovers ----------------
        realBox = wx.StaticBox(self, -1, "Real operations (affects only feature weighting)", size=(500,-1))
        realBoxSizer = wx.StaticBoxSizer(realBox, wx.VERTICAL)
        realBoxFlexGridSizer = wx.FlexGridSizer(0, 2, 5, 5)
        realBoxSizer.Add(realBoxFlexGridSizer, 0)
        sizer.Add(realBoxSizer, 0, wx.ALL | wx.EXPAND, 10)
        
        # simulated binary crossover
        self.sbxCrossover = wx.CheckBox(self, -1, "SBX Crossover", \
            name = "SBXcrossover")
        self.sbxCrossover.SetValue(True)
        self.sbxCrossover.Disable()
        realBoxFlexGridSizer.Add(self.sbxCrossover, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 5)
        
        self.sbxCrossoverEta = FS.FloatSpin(self, -1, min_val=0.01, max_val=10.0, \
            increment=0.01, value=1.0)
        self.sbxCrossoverEta.SetFormat("%f")
        self.sbxCrossoverEta.SetDigits(2)
        self.sbxCrossoverEta.Disable()
        self.sbxCrossoverEta.SetToolTipString("eta the amount of exploration "\
            "OUTSIDE the parents in BLX-alpha notation")
        realBoxFlexGridSizer.Add(self.sbxCrossoverEta, 0, wx.LEFT | wx.TOP | wx.RIGHT, 5)

        self.weightingWidgets.append(self.sbxCrossover)
        self.AddChildToParent(self.sbxCrossover, self.sbxCrossoverEta)
        
        # segment crossover
        self.segmentCrossover = wx.CheckBox(self, -1, "Segment Crossover", \
            name = "segmentCrossover")
        self.segmentCrossover.Disable()
        realBoxFlexGridSizer.Add(self.segmentCrossover, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 5)

        self.segmentCrossoverAlpha = FS.FloatSpin(self, -1, min_val=0.01, max_val=10.0, \
            increment=0.01, value=1.0)
        self.segmentCrossoverAlpha.SetFormat("%f")
        self.segmentCrossoverAlpha.SetDigits(2)
        self.segmentCrossoverAlpha.Disable()
        self.segmentCrossoverAlpha.SetToolTipString("alpha the amount of exploration "\
            "OUTSIDE the parents in BLX-alpha notation")
        realBoxFlexGridSizer.Add(self.segmentCrossoverAlpha, 0, wx.LEFT | wx.TOP | wx.RIGHT, 5)
        
        self.weightingWidgets.append(self.segmentCrossover)
        self.AddChildToParent(self.segmentCrossover, self.segmentCrossoverAlpha)
        
        # hypercube crossover
        self.hypercubeCrossover = wx.CheckBox(self, -1, "Hypercube Crossover", \
            name = "hypercubeCrossover")
        self.hypercubeCrossover.Disable()
        realBoxFlexGridSizer.Add(self.hypercubeCrossover, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND, 5)
        
        self.hypercubeCrossoverAlpha = FS.FloatSpin(self, -1, min_val=0.01, max_val=10.0, \
            increment=0.01, value=1.0)
        self.hypercubeCrossoverAlpha.SetFormat("%f")
        self.hypercubeCrossoverAlpha.SetDigits(2)
        self.hypercubeCrossoverAlpha.Disable()
        self.hypercubeCrossoverAlpha.SetToolTipString("alpha the amount of exploration "\
            "OUTSIDE the parents in BLX-alpha notation")
        realBoxFlexGridSizer.Add(self.hypercubeCrossoverAlpha, 0, wx.LEFT | wx.TOP | wx.RIGHT, 5)
        
        self.weightingWidgets.append(self.hypercubeCrossover)
        self.AddChildToParent(self.hypercubeCrossover, self.hypercubeCrossoverAlpha)
        
        # bind the EVT_CHECKBOX to the CheckBoxes
        self.BindEvent(wx.EVT_CHECKBOX, self.OnCheckBox, \
            [self.nPointCrossover,
             self.uniformCrossover,
             self.sbxCrossover,
             self.segmentCrossover,
             self.hypercubeCrossover])



# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, and Karl MacMillan
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import wx
from wx.lib import dialogs
from os import path
from types import *
from gamera import util
from gamera.config import config
import sys
import datetime

config.add_option(
   "", "--default-dir", default=".",
   help="[gui] The default directory when opening files")

colors = (wx.Colour(0xbc, 0x2d, 0x2d), wx.Colour(0xb4, 0x2d, 0xbc),
          wx.Colour(0x2d, 0x34, 0xbc), wx.Colour(0x2d, 0xbc, 0x2d),
          wx.Colour(0x2d, 0xbc, 0xbc), wx.Colour(0xbc, 0xb7, 0x2d),
          wx.Colour(0xbc, 0x88, 0x2d), wx.Colour(0x6e, 0x00, 0xc7))

color_number = 0
def get_color(number):
   global color_number
   if isinstance(number, wx.Colour):
      return number
   if util.is_sequence(number):
      if len(number) == 3:
         return wx.Colour(*tuple(number))
      else:
         number = None
   if type(number) != IntType:
      number = color_number
      color_number += 1
   return colors[number & 0x7] # mod 8

# Displays a message box
def message(message):
   if message.count("\n") > 4:
      dlg = dialogs.ScrolledMessageDialog(
         None, message, "Message")
   else:
      dlg = wx.MessageDialog(
         None, message, "Message",
         wx.OK|wx.ICON_INFORMATION)
   dlg.ShowModal()
   dlg.Destroy()

def are_you_sure_dialog(message, parent=None):
   dlg = wx.MessageDialog(
      parent, message, "Are you sure?",
      wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
   result = dlg.ShowModal()
   dlg.Destroy()
   return result == wx.ID_YES

def build_menu(parent, menu_spec):
   global menu_item_id
   menu = wx.Menu()
   for name, func in menu_spec:
      if util.is_sequence(func):
         menu_item_id = wx.NewId()
         menu.AppendMenu(menu_item_id, name, build_menu(parent, func))
      elif name == None:
         menu.AppendSeparator()
      else:
         menu_item_id = wx.NewId()
         menu.Append(menu_item_id, name)
         wx.EVT_MENU(parent, menu_item_id, func)
   return menu

NUM_RECENT_FILES = 9
class FileDialog(wx.FileDialog):
   last_directory = None
   
   def __init__(self, parent, extensions="*.*", multiple=0):
      cls = self.__class__
      if cls.last_directory is None:
         cls.last_directory = path.abspath(config.get("default_dir"))
      if multiple:
         self._flags |= wx.FD_MULTIPLE
         self._multiple = True
      else:
         self._multiple = False
      wx.FileDialog.__init__(
         self, parent, "Choose a file",
         cls.last_directory, "", str(extensions), style=self._flags)
      self.extensions = extensions

   def show(self):
      if wx.VERSION < (2, 8):
         self.SetStyle(self._flags)
      cls = self.__class__
      result = self.ShowModal()
      self.Destroy()
      if result == wx.ID_CANCEL:
         return None
      if self._multiple:
         filenames = self.GetPaths()
         cls.last_directory = path.dirname(filenames[0])
         return filenames
      else:
         filename = self.GetPath()
         cls.last_directory = path.dirname(filename)
         return filename

class OpenFileDialog(FileDialog):
   _flags = wx.FD_OPEN

class SaveFileDialog(FileDialog):
   _flags = wx.SAVE|wx.FD_OVERWRITE_PROMPT

def open_file_dialog(parent, extensions="*.*", multiple=0):
   return OpenFileDialog(parent, extensions, multiple).show()

def save_file_dialog(parent, extensions="*.*"):
   return SaveFileDialog(parent, extensions).show()

def directory_dialog(parent, create=1):
   last_directory = config.get("default_dir")
   if create:
      style = wx.DD_NEW_DIR_BUTTON
   else:
      style = 0
   dlg = wx.DirDialog(parent, "Choose a directory", last_directory, style)
   if dlg.ShowModal() == wx.ID_OK:
      filename = dlg.GetPath()
      dlg.Destroy()
      return filename
   return None

class ProgressBox:
   def __init__(self, message, length=1, numsteps=0):
      assert util.is_string_or_unicode(message)
      self.progress_box = wx.ProgressDialog(
          "Progress", message, 100,
         style=wx.PD_APP_MODAL|wx.PD_ELAPSED_TIME|wx.PD_REMAINING_TIME|wx.PD_AUTO_HIDE)
      self.done = 0
      self._num = 0
      if length == 0:
         self._den = 1
      else:
         self._den = length
      self._numsteps = numsteps
      self._lastupdate = 0
      self._lasttime = datetime.datetime.now()
      wx.BeginBusyCursor()

   def __del__(self):
      if not self.done:
         self.done = 1
         self.progress_box.Destroy()
         wx.EndBusyCursor()

   def add_length(self, l):
      self._den += l

   def set_length(self, l):
      self._den = l

   def step(self):
      self._num += 1
      self.update(self._num, self._den)

   def update(self, num, den):
      if not self.done:
         if num >= den or num == self._lastupdate:
            self.done = True
            wx.EndBusyCursor()
            #self.progress_box.Destroy()
            wx.CallAfter(self.progress_box.Destroy)
         elif 0 == self._numsteps or \
                (den/(num-self._lastupdate) <= self._numsteps):
            # make sure that the progress bar is not updated too often
            #td = datetime.datetime.now() - self._lasttime
            #if (td.days*86400 + td.seconds + td.microseconds/100000.0) <= 0.3:
            #    return
            self.progress_box.Update(min(100, int((float(num) / float(den)) * 100.0)))
            self._lastupdate = num
            self._lasttime = datetime.datetime.now()

   def kill(self):
      self.update(1, 1)

######################################################################
# DOCUMENTATION DISPLAY
#
# If available, we use docutils to format the doc strings from
# reStructuredText into HTML, and display it with a wx.html.HtmlWindow.
# Otherwise, we just default to displaying it as text.
# Since reStructuredText -> HTML conversion is quite slow, we
# cache the HTML chunks based on the md5sum of the docstring.
try:
   import docutils.core
   import docutils.parsers.rst
except ImportError, e:
   # If we don't have docutils, we just wrap the docstring
   # in <pre> tags
   def docstring_to_html(docstring):
      return "<pre>%s</pre>" % docstring
else:
   # Some docstrings may contain SilverCity ".. code::" blocks.
   # Since wx.html.HtmlWindow does not support CSS, it therefore
   # can not properly handle SilverCity's syntax coloring.  So instead,
   # we create a "dummy" code block handler that simply uses <pre>
   def code_block(name, arguments, options, content, lineno,
                  content_offset, block_text, state, state_machine ):
      html = '\n<pre>%s</pre>\n' % "\n".join(content)
      raw = docutils.nodes.raw('', html, format = 'html')
      return [raw]
   code_block.arguments = (1,0,0)
   code_block.options = {'language' : docutils.parsers.rst.directives.unchanged }
   code_block.content = 1
   docutils.parsers.rst.directives.register_directive( 'code', code_block )

   def docstring_to_html(docstring):
      try:
         corrected = docstring.replace("*args", "\*args")
         corrected = corrected.replace("**kwargs", "\*\*kwargs")
         html = docutils.core.publish_string(corrected, writer_name="html")
      except Exception, e:
         html = '''<pre>%s</pre><br/<br/><font size=1><pre>%s</pre></font>''' % (docstring, str(e))
      return html.decode("utf-8")



import py.test

from gamera.core import *
init_gamera()

#
# Tests for nearest neighbor finding with kd-trees
#

from gamera.kdtree import *

#
# input parameter check
#
def test_wrongparams():
    def _kdnode_input(coord):
        node = KdNode(coord)
    def _kdtree_input(nodes):
        tree = KdTree(nodes)
    # all point coordinates must be numeric
    py.test.raises(Exception, _kdnode_input, [1,2,"a"])
    py.test.raises(Exception, _kdnode_input, [])
    # all nodes must be KdNode's and of same dimension
    py.test.raises(Exception, _kdtree_input, [KdNode([1,2]),KdNode([2,3,4])])
    py.test.raises(Exception, _kdtree_input, [KdNode([1,2]),[2,3]])
    py.test.raises(Exception, _kdtree_input, None)
    py.test.raises(Exception, _kdtree_input, [])


#
# k nearest neighbor searches
#
def test_nearest_neighbors():
    points = [(1,1), (2,1), (1,3), (2,4), (3,4),
              (7,2), (8,3), (8,4), (7,5), (7,5)]
    nodes = [KdNode(p) for p in points]
    tree = KdTree(nodes)
    assert [7,2] == tree.k_nearest_neighbors([5.5,3],1)[0].point
    assert [[2,4], [3,4]] == \
        [n.point for n in tree.k_nearest_neighbors([2,4],2)]
    assert [[7,5],[7,5],[8,4]] == \
        [n.point for n in tree.k_nearest_neighbors([7,5],3)]
    assert [[8,4],[8,3],[7,5]] == \
        [n.point for n in tree.k_nearest_neighbors([8,4],3)]
    assert [[3,4],[2,4],[1,3],[2,1],[1,1],[7,5]] == \
        [n.point for n in tree.k_nearest_neighbors([3,4],6)]

#
# tests with different distance measures
#
def test_distance_metrics():
    points = [(1,4), (2,4), (1,5), (3,6), (8,9),
              (3.2,4.2), (4,4), (5,5), (3.8,6), (8,3)]
    nodes = [KdNode(p) for p in points]
    tree = KdTree(nodes)
    tree.set_distance(0)
    assert [[5,5], [3.8,6], [3.2,4.2]] == \
        [n.point for n in tree.k_nearest_neighbors([5,6],3)]
    tree.set_distance(1)
    assert [[5,5], [3.8,6], [3,6]] == \
        [n.point for n in tree.k_nearest_neighbors([5,6],3)]
    tree.set_distance(2,[1.0,0.5])
    assert [[5,5], [3.8,6], [4,4]] == \
        [n.point for n in tree.k_nearest_neighbors([5,6],3)]
    tree.set_distance(0,[0.5,1.0])
    assert [[3.8,6], [3,6], [5,5]] == \
        [n.point for n in tree.k_nearest_neighbors([5,6],3)]

#
# tests with search predicate
#
def test_search_predicate():
    class predicate(object):
        def __init__(self, point):
            self.point = point
        def __call__(self, node):
            return (self.point[1] > node.point[1])
    points = [(1,4), (2,4), (1,5), (3,6), (8,9),
              (3.2,4.2), (4,4), (5,5), (3.8,6), (8,3)]
    nodes = [KdNode(p) for p in points]
    tree = KdTree(nodes)
    assert [[5,5], [4,4]] == \
        [n.point for n in tree.k_nearest_neighbors([5,6],2,predicate([5,6]))]
    assert 0 == len(tree.k_nearest_neighbors([5,6],2,predicate([1,2])))



#
# Copyright (C) 2012 Tobias Bolten
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import wx

from gamera.gui.gaoptimizer.OptimizerFrame import *

#-------------------------------------------------------------------------------
class OptimizerApp(wx.App):
#-------------------------------------------------------------------------------
    #---------------------------------------------------------------------------
    def OnInit(self):
    #---------------------------------------------------------------------------
        frame = OptimizerFrame(None, -1, "Biollante - GA Optimization for kNN")
        frame.Show()
        self.SetTopWindow(frame)

        return True

#-------------------------------------------------------------------------------
if __name__ == "__main__":
#-------------------------------------------------------------------------------
    app = OptimizerApp()
    app.MainLoop()



import sys

from gamera.core import *
init_gamera()

# Load filename specified on the command line.
# sys.argv is the Pythonic way to access commandline arguments.
# load_image is a Gamera function to load a TIFF or PNG file.
image = load_image(sys.argv[-1])

# The variable 'image' now is a reference to the image

# Convert the image to onebit using `otsu_threshold`
onebit = image.otsu_threshold()

# Save the result to a PNG file
onebit.save_PNG("output.png")
